
/* ================================================================= */
/*                                                                   */
/*		LOW-LEVEL OnCE interface                             */
/*                                                                   */
/* ================================================================= */

#ifdef __KERNEL__
# define PRINTK(a...)	printk(a)
#else
# include <stdio.h>
# define PRINTK(a...)	fprintf(stderr, ##a)
#endif
#define __NO_VERSION__
#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/types.h>

#include <asm/io.h>
#include <asm/segment.h>

#include <linux/dtc2.h>
#include "OnCE.h"

#define DEBUG	1

int DSPreset(void);

int in_debug_mode = 0;

u_long bk_hi=0,bk_lo=0,bk_cnt=0;
int   isbk=0;

/*
	01	bf	break on fetch
	02	bpf	break on P read/fetch
	03	bpa	break on P access
	04	bfe	break on executed fetches
	05	bpw	break on P write
	06	bpr	break on P read
	07	bp	break on P read/write
	09	bxw	break on X write
	0a	bxr	break on X read
	0b	bx	break on X read/write
	0d	byw	break on Y write
	0e	byr	break on Y read
	0f	by	break on Y read/write
*/

/* ================================================================= */
/*
  Try to guess if there was an OnCE request - look at ACK,DSCK,DSI
	if DSCK:DSI != 00 they are outputs, DSP not in debug mode
	if DSCK:DSI == 00 and ACK=0 the DSP is in normal run mode
			*OR* ACK was canceled not in debug mode (beware!)
	if DSCK:DSI == 00 and ACK=1 the DSP is in debug mode waiting
			for service *OR* someone forgot to clear ACK
			after a debug session (beware!)
*/
int OnCEreq(void)
{
  return((inb(OnCEstat)&7)==4);
}


int OnCErunmode(void)
{
  return(inb(OnCEstat)&3);
}


int enterOnCE(int reset)
{
 register u_int t __asm__("cx") = 100000;

  if (in_debug_mode) return(-1);
  if (reset || !OnCEreq()) {
    outb(1,OnCEstat);			/* set DR=0, clear ACK */
    if (reset) {			/* reset processor */
      DSPreset();
     }
    while (((inb(OnCEstat)&4)==0)&&(--t));
    outb(0,OnCEstat);			/* set DR=1, clear ACK */
    if (t==0) return(1);
   }
  else outb(0,OnCEstat);		/* clear request */
  in_debug_mode=1;
  return(0);
}

/* ================================================================= */

int send_command(register int n)
{
 register u_short t __asm__("cx") = 65535;

	__asm__ __volatile__ (
		"movl %0,%%eax\n\t"
		"movl %1,%%edx\n\t"
		"outb %%al,%%dx\n\t"	/* data latched into shift reg */
		"xorb %%al,%%al\n\t"
		"incl %%edx\n\t"
#ifdef __KERNEL__
		"cli\n\t"
#endif
/* 8 clks at PCCLK/4 -> 32 PCCLK cycles (1.92..3.84 us) */
		"outb %%al,%%dx\n\t"	/* start 8 clock cycles */
		"subl $2,%%edx\n"
	"1:	inb %%dx,%%al\n\t"	/* wait for end of clock sequence */
		"testb $8,%%al\n\t"
		"je 1b\n\t"
#ifdef __KERNEL__
		"sti\n"
#endif
	:
	: "g"(n), "n"(OnCEdat)
	: "eax", "edx", "memory" );

/* command has always an ACK, look for it */
	while (((inb(OnCEstat)&4)==0)&&(--t));
	return(t==0);
}

/* ================================================================= */

int send_data24(u_long v,int ack)
{
  register u_int t __asm__("ecx");

	__asm__ __volatile__ (
		"movl %0,%%eax\n\t"
		"rorl $16,%%eax\n\t"		/* eax=mmll00hh */
		"movl %1,%%edx\n\t"
		"outb %%al,%%dx\n\t"
		"incl %%edx\n\t"
#ifdef __KERNEL__
		"cli\n\t"
#endif
		"outb %%al,%%dx\n\t"		/* clock */
		"subl $2,%%edx\n"
	"1:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 1b\n\t"
		"roll $8,%%eax\n\t"		/* eax=ll00xxmm */
		"incl %%edx\n\t"
		"outb %%al,%%dx\n\t"
		"incl %%edx\n\t"
		"outb %%al,%%dx\n\t"		/* clock */
		"subl $2,%%edx\n"
	"2:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 2b\n\t"
		"roll $8,%%eax\n\t"		/* eax=00xxxxll */
		"incl %%edx\n\t"
		"outb %%al,%%dx\n\t"
		"decl %%edx\n\t"
		"xorb %%al,%%al\n\t"
		"outb %%al,%%dx\n\t"
		"addl $2,%%edx\n\t"
		"outb %%al,%%dx\n\t"
		"subl $2,%%edx\n"
	"3:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 3b\n\t"
#ifdef __KERNEL__
		"sti\n\t"
#endif
	:
	: "g"(v), "n"(OnCEdat)
	: "eax", "edx", "memory" );

	if (!ack) return(0);
	t=20000;
	while (((inb(OnCEstat)&4)==0)&&(--t));
	return(t==0);
}

/* ================================================================= */

long get_data24(int ack)
{
  register u_int t __asm__("ecx");
  register u_long v;

	__asm__ __volatile__ (
		"xorl %%eax,%%eax\n\t"
		"movl %1,%%edx\n\t"
		"outb %%al,%%dx\n\t"
		"incl %%edx\n\t"
#ifdef __KERNEL__
		"cli\n\t"
#endif
		"outb %%al,%%dx\n\t"		/* clock */
		"subl $2,%%edx\n"
	"1:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 1b\n\t"
		"incl %%edx\n\t"
		"inb  %%dx,%%al\n\t"		/* eax=000000hh */
		"roll $8,%%eax\n\t"		/* eax=0000hh00 */
		"incl %%edx\n\t"
		"outb %%al,%%dx\n\t"		/* clock */
		"subl $2,%%edx\n"
	"2:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 2b\n\t"
		"incl %%edx\n\t"
		"inb  %%dx,%%al\n\t"		/* eax=0000hhmm */
		"roll $8,%%eax\n\t"		/* eax=00hhmm00 */
		"decl %%edx\n\t"
		"outb %%al,%%dx\n\t"
		"addl $2,%%edx\n\t"
		"outb %%al,%%dx\n\t"		/* clock */
		"subl $2,%%edx\n"
	"3:	 inb  %%dx,%%al\n\t"
		"testb $8,%%al\n\t"
		"je 3b\n\t"
#ifdef __KERNEL__
		"sti\n\t"
#endif
		"incl %%edx\n\t"
		"inb  %%dx,%%al\n\t"		/* eax=00hhmmll */
		"movl %%eax,%0\n"
	: "=g"(v)
	: "n"(OnCEdat)
	: "eax", "edx", "memory" );

	if (ack) {
		t=20000;
		while (((inb(OnCEstat)&4)==0)&&(--t));
		if (t==0) return(-1l);
	}
	return(v);		/* always positive if no error */
}


/* ================================================================= */
/* send command and read data - do not check for ACK */

long OnCErdat(u_char reg)
{
  if (send_command(reg|O_read)) return(-1l);
  return(get_data24(0));
}

/* send command and write data with/without ACK */

int OnCEwdat(int reg,u_long v,int ack)
{
  if (send_command(reg|O_write)) return(1);
  return(send_data24(v,ack));
}


/* ================================================================= */
/*                                                                   */
/*                                                                   */
/* ================================================================= */


int begin_debug_run(OnCEregs *o,int reset)
{
 int i,err;

#ifdef DEBUG
  PRINTK("Enter OnCE, status=%02x\n",inb(OnCEstat)&15);
#endif

  if ((err=enterOnCE(reset))) {
    if (err<0) PRINTK("OnCE already active");
     else PRINTK("Timeout on DR ACK");
    return(1);
   }
#ifdef DEBUG
  PRINTK("Inside OnCE, status=%02x\n",inb(OnCEstat)&15);
#endif

  if ((o->PDBR=OnCErdat(O_OPDBR))==-1l) return(2);
  if ((o->PILR=OnCErdat(O_OPILR))==-1l) return(3);
  if ((o->PABFR=OnCErdat(O_OPABFR))==-1l) return(4);
  if ((o->PABDR=OnCErdat(O_OPABDR))==-1l) return(5);
  for (i=0; i<5; i++)
    if ((o->PABfifo[i]=OnCErdat(O_FIFO))==-1l) return(6);
  if ((o->SCR=OnCErdat(O_OSCR))==-1l) return(7);

  return(0);
}


int end_debug_go(OnCEregs *o,u_long a)	/* if a==0 no jump */
{
  OnCEwdat(O_OSCR,0,0);		/* no trace mode */
  if (WritePDBAck(a? O_jmplong:o->PILR)) return(10);
  if (WritePDBgoex(a? a:o->PDBR)) return(11);	/* address right aligned (M,L) */
  in_debug_mode=0;

#ifdef DEBUG
  PRINTK("Exit OnCE");
  if (a) PRINTK(" and jump %04lx",a);
  PRINTK(", status=%02x\n",inb(OnCEstat)&15);
#endif
  return(0);
}


int OnCEtrace(OnCEregs *o,u_long n,u_long a)
{
 int i,stat;
 u_long v;

  OnCEwdat(O_OTC,n-1,0);
  OnCEwdat16(O_OSCR,o->SCR|O_OSCR_TME,0);
  v=OnCErdat(O_OSCR);
  if (WritePDBAck(a==0xffff? o->PILR:O_jmplong)) return(10);
  if (WritePDBgoex(a==0xffff? o->PDBR:a)) return(11);

  while ((stat=(inb(OnCEstat)&7))!=4) {
	current->state = TASK_INTERRUPTIBLE;
	current->timeout = jiffies + 2;
	schedule();
  }
  if ((o->SCR=OnCErdat(O_OSCR))==-1l) return(7);
  if ((o->PDBR=OnCErdat(O_OPDBR))==-1l) return(2);
  if ((o->PILR=OnCErdat(O_OPILR))==-1l) return(3);
  if ((o->PABFR=OnCErdat(O_OPABFR))==-1l) return(4);
  if ((o->PABDR=OnCErdat(O_OPABDR))==-1l) return(5);
  for (i=0; i<5; i++)
    if ((o->PABfifo[i]=OnCErdat(O_FIFO))==-1l) return(6);

  return(0);
}


int OnCEbrkgo(OnCEregs *o)
{
 volatile int i,stat;
 u_long v;

  v=OnCErdat(O_OSCR);
  OnCEwdat(O_OMBC,bk_cnt-1,0);
  OnCEwdat(O_OMLLR,bk_lo,0);
  OnCEwdat(O_OMULR,bk_hi,0);
  OnCEwdat16(O_OSCR,isbk,0);
  if (WritePDBAck(o->PILR)) return(10);
  if (WritePDBgoex(o->PDBR)) return(11);

  while ((stat=(inb(OnCEstat)&7))!=4) {
	current->state = TASK_INTERRUPTIBLE;
	current->timeout = jiffies + 2;
	schedule();
  }
  if ((o->SCR=OnCErdat(O_OSCR))==-1l) return(7);
  OnCEwdat(O_OSCR,v,0);
  if ((o->PDBR=OnCErdat(O_OPDBR))==-1l) return(2);
  if ((o->PILR=OnCErdat(O_OPILR))==-1l) return(3);
  if ((o->PABFR=OnCErdat(O_OPABFR))==-1l) return(4);
  if ((o->PABDR=OnCErdat(O_OPABDR))==-1l) return(5);
  for (i=0; i<5; i++)
    if ((o->PABfifo[i]=OnCErdat(O_FIFO))==-1l) return(6);

  return(0);
}


int OnCEdebgo(OnCEregs *o)
{
 volatile int i,stat;
 u_long v;

  OnCEwdat(O_OSCR,0,0);		/* no trace mode */
  if (WritePDBAck(o->PILR)) return(10);
  if (WritePDBgoex(o->PDBR)) return(11);

  while ((stat=(inb(OnCEstat)&7))!=4) {
	current->state = TASK_INTERRUPTIBLE;
	current->timeout = jiffies + 2;
	schedule();
  }
  if ((o->SCR=OnCErdat(O_OSCR))==-1l) return(7);
  if ((o->PDBR=OnCErdat(O_OPDBR))==-1l) return(2);
  if ((o->PILR=OnCErdat(O_OPILR))==-1l) return(3);
  if ((o->PABFR=OnCErdat(O_OPABFR))==-1l) return(4);
  if ((o->PABDR=OnCErdat(O_OPABDR))==-1l) return(5);
  for (i=0; i<5; i++)
    if ((o->PABfifo[i]=OnCErdat(O_FIFO))==-1l) return(6);

  return(0);
}


/* ------------------------------------------------------------------- */


/* User registers */
u_long OnCErreg(int r)		/* r = register NUM */
{
  Execute1i(O_GmovR|(r<<8));
  return(ReadGDB());
}

int OnCEwreg(int r,u_long v)	/* r = register NUM */
{
 u_long opr;

  opr=r;
  if (r<reg_M0) {
    if (r>=reg_N0) opr+=0x18;
     else if (r>=reg_R0) opr+=0x10;
      else if (r>=reg_A0) opr+=8;
    Execute2i1(O_ImovR|(opr<<16));
   }
  else {
    Execute2i1(O_ImovM|r);
   }
  return(Execute2i2(v));
}


int OnCEgetallreg(DSPregs *r,u_long mask)
{
 int i;

  if (mask&msk_X0) r->X0=OnCErreg(reg_X0);
  if (mask&msk_X1) r->X1=OnCErreg(reg_X1);
  if (mask&msk_Y0) r->Y0=OnCErreg(reg_Y0);
  if (mask&msk_Y1) r->Y1=OnCErreg(reg_Y1);
  if (mask&msk_A2) r->A2=OnCErreg(reg_A2);
  if (mask&msk_A1) r->A1=OnCErreg(reg_A1);
  if (mask&msk_A0) r->A0=OnCErreg(reg_A0);
  if (mask&msk_B2) r->B2=OnCErreg(reg_B2);
  if (mask&msk_B1) r->B1=OnCErreg(reg_B1);
  if (mask&msk_B0) r->B0=OnCErreg(reg_B0);
  for (i=0; i<8; i++) {
    if (mask&msk_RNM(i)) {
      r->R[i]=OnCErreg(i+0x10);
      r->N[i]=OnCErreg(i+0x18);
      r->M[i]=OnCErreg(i+0x20);
     }
   }
  if (mask&msk_SR) r->SR=OnCErreg(reg_SR);
  if (mask&msk_OMR) r->OMR=OnCErreg(reg_OMR);
  if (mask&msk_LA) r->LA=OnCErreg(reg_LA);
  if (mask&msk_LC) r->LC=OnCErreg(reg_LC);
  if (mask&msk_SP) r->SP=OnCErreg(reg_SP);
  return(0);
}


int OnCErmem(char space,u_long a,int n,u_long *buf)
{
 u_long r0;		/* save R0 value */
 u_long Smove,Mask24;

  if (n==0) return(0);
  Mask24=0xffffffl;
  switch(space) {
    case 'x': Smove = O_movR0p+0x40; break;
    case 'y': Smove = O_movR0p+0x80; break;
    case 'p': Smove = O_movR0p+0x00; break;
    default:  return(1);
   }
  r0=OnCErreg(reg_R0);
  Execute2i1(O_moviR0);			/* move #aaaa,r0 */
  Execute2i2(a);
  Execute1i(Smove);			/* move S:(r0)+,X:OGDB */
  *buf++ = ReadGDB()&Mask24; n--;
  while (n--) {
    NOPgo(); *buf++ = ReadGDB()&Mask24;
   }
  OnCEwreg(reg_R0,r0);
  return(0);
}



int OnCEwmem(char space,u_long a,int n,u_long *buf)
{
 u_long r0,x0;		/* save R0,X0 value */
 u_long Smove;

  if (n==0) return(0);
  switch(space) {
    case 'x': Smove = 0x445800l; break;		/* move x0,X:(r0)+ */
    case 'y': Smove = 0x4c5800l; break;		/* move x0,Y:(r0)+ */
    case 'p': Smove = 0x075884l; break;		/* move x0,P:(r0)+ */
    default:  return(1);
   }
  r0=OnCErreg(reg_R0);
  x0=OnCErreg(reg_X0);
  Execute2i1(O_moviR0);			/* move #aaaa,r0 */
  Execute2i2(a);
  Execute2i1(O_moviX0);			/* move #nnnn,x0 */
  Execute2i2(*buf++);
  Execute1i(Smove); n--;		/* move x0,d:(r0)+ */
  while (n--) {
    Execute2i1(O_moviX0);		/* move #nnnn,x0 */
    Execute2i2(*buf++);
    Execute1i(Smove);	 		/* move x0,d:(r0)+ */
   }
  OnCEwreg(reg_X0,x0);
  OnCEwreg(reg_R0,r0);
  return(0);
}


int OnCEfillmem(char space,u_long a,int n,u_long v)
{
 u_long r0,x0;		/* save R0,X0 value */
 u_long Smove;

  if (n==0) return(0);
  switch(space) {
    case 'x': Smove = 0x445800l; break;		/* move x0,X:(r0)+ */
    case 'y': Smove = 0x4c5800l; break;		/* move x0,Y:(r0)+ */
    case 'p': Smove = 0x075884l; break;		/* move x0,P:(r0)+ */
    default:  return(1);
   }
  r0=OnCErreg(reg_R0);
  x0=OnCErreg(reg_X0);
  Execute2i1(O_moviR0);			/* move #aaaa,r0 */
  Execute2i2(a);
  Execute2i1(O_moviX0);			/* move #nnnn,x0 */
  Execute2i2(v);
  Execute1i(Smove); n--;		/* move x0,d:(r0)+ */
  while (n--) {
    NOPgo();
   }
  OnCEwreg(reg_X0,x0);
  OnCEwreg(reg_R0,r0);
  return(0);
}


/* ================================================================= */

