
/* ==================================================================== */
/*			OnCE.h	2/5/95					*/
/* ==================================================================== */

#ifndef _DTC2_ONCE_H
#define _DTC2_ONCE_H

#ifdef  CONFIG_DTC2

#include <linux/dtc2.h>

#undef X0
#undef X1
#undef Y0
#undef Y1
#undef A0
#undef A1
#undef A2
#undef B0
#undef B1
#undef B2
#undef SR
#undef OMR
#undef LA
#undef LC
#undef SP

/* ----------------------------------------------------------------- */

#define OnCEstat	ONCE
#define OnCEdat		ONCE+1		/* write data,no clock */
#define OnCEclk		ONCE+2		/* write=clk */

typedef struct _OnCEregs {
/* 00000 */	u_long SCR;		/* 16bit R/W */
/* 00001 */	u_long MBCR;		/* 24bit R/W */
/* 00011 */	u_long TR;		/* 24bit R/W */
/* 00110 */	u_long MULR;		/* 16bit R/W */
/* 00111 */	u_long MLLR;		/* 16bit R/W */
/* 01000 */	u_long GDBR;		/* 24bit R/O */
/* 01001 */	u_long PDBR;		/* 24bit R/W */
/* 01010 */	u_long PABFR;		/* 16bit R/O */
/* 01011 */	u_long PILR;		/* 24bit R/O */
/* 01100 */	u_long MBCC;            /* clear MBC */
/* 01110 */	u_long TC;		/* clear TR  */
/* 10001 */	u_long PABfifo[5];	/* 16bit R/O */
/* 10011 */	u_long PABDR;		/* 16bit R/O */
} OnCEregs;

typedef struct _DSPregs {
	u_long	X0,X1,Y0,Y1;
	u_char 	A2;
	u_long	A1,A0;
	u_char 	B2;
	u_long	B1,B0;
	u_short	R[8],N[8],M[8];
	u_short	SR,OMR,LA,LC;
	u_short	SP;
} DSPregs;


/* OnCE command register - bits 0-4=register no. */
#define O_read		0x80
#define O_write		0x00
#define O_go		0x40
#define O_nogo		0x00
#define O_ex		0x20
#define O_noex		0x00

/* OnCE registers */
#define O_OSCR		0x00	/* status/control register */
#define O_OMBC		0x01	/* memory breakpoint counter */
#define O_OTC		0x03	/* trace counter */
#define O_OMULR		0x06	/* memory upper limit register */
#define O_OMLLR		0x07	/* memory lower limit register */
#define O_OGDBR		0x08	/* global data bus register */
#define O_OPDBR		0x09	/* program data bus register */
#define O_OPABFR	0x0A	/* PAB register for fetch */
#define O_OPILR		0x0B	/* pipeline instruction latch register */
#define O_OCMBC		0x0C	/* clear memory breakpoint counter */
#define O_OCTC		0x0E	/* clear trace counter */
#define O_FIFO		0x11
#define O_OPABDR	0x13	/* PAB register for decode */
#define O_noreg		0x1F

/* OnCE status/control register 0x00 */
/* 16 bits, MSB aligned */
#define O_OSCR_TO	0x400	/* trace occurrence, R/O */
#define O_OSCR_MBO	0x200	/* memory breakpoint occurrence, R/O */
#define O_OSCR_SWO	0x100	/* SW debug occurrence, R/O */
#define O_OSCR_TME	0x10	/* Trace mode enable, R/W */
#define O_OSCR_BC	0x0f	/* mask for breakpoint mode */

/* breakpoint modes, bits 0-3 of SCR */
#define O_nobrk		0x0
#define O_brk_fetch	0x1	/* fetch+aborted instructions */
#define O_brk_readPf	0x2	/* read P+fetch */
#define O_brk_accessP	0x3	/* read+write+fetch P */
#define O_brk_xfetch	0x4	/* executed fetches */
#define O_brk_writeP	0x5	/* write P */
#define O_brk_readP	0x6	/* read P, no fetch */
#define O_brk_rwP	0x7	/* read+write P */
#define O_brk_writeX	0x9
#define O_brk_readX	0xa
#define O_brk_rwX	0xb
#define O_brk_writeY	0xd
#define O_brk_readY	0xe
#define O_brk_rwY	0xf

#define O_GmovR		0x08C03C	/* movep reg,X:OGDB */
#define O_RmovG		0x08403C	/* movep X:OGDB,reg */
#define O_ImovR		0x40f400	/* move #>$xxxxxx,reg */
#define O_ImovM		0x05f400	/* move #>$xxxxxx,reg */

/* OR with register - order:
	0x400	x0,x1,y0,y1
	0x800	a0,b0,a2,b2,a1,b1,a,b
	0x1000	r0..r7
	0x1800	n0..n7
	0x2000	m0..m7
	0x2800	-------
	0x3900	sr,omr,sp,ssh,ssl,la,lc
*/
#define reg_X0		0x04		/* 0x00000002 */
#define reg_X1		0x05		/* 0x00000001 */
#define reg_Y0		0x06		/* 0x00000008 */
#define reg_Y1		0x07		/* 0x00000004 */
#define reg_A0		0x08		/* 0x00000040 */
#define reg_B0		0x09		/* 0x00000200 */
#define reg_A2		0x0a		/* 0x00000010 */
#define reg_B2		0x0b		/* 0x00000080 */
#define reg_A1		0x0c		/* 0x00000020 */
#define reg_B1		0x0d		/* 0x00000100 */
#define reg_A 		0x0e
#define reg_B 		0x0f
#define reg_R0		0x10
#define reg_R(r)	(0x10+(r))	/* 0x0003FC00 */
#define reg_N0		0x18
#define reg_N(r)	(0x18+(r))	/* 0x0003FC00 */
#define reg_M0		0x20
#define reg_M(r)	(0x20+(r))	/* 0x0003FC00 */
#define reg_SR		0x39		/* 0x00040000 */
#define reg_OMR		0x3a		/* 0x00080000 */
#define reg_SP		0x3b		/* 0x00100000 */
#define reg_SSH		0x3c		/* 0x00200000: do not read! */
#define reg_SSL		0x3d		/* 0x00400000: do not read! */
#define reg_LA		0x3e		/* 0x00800000 */
#define reg_LC		0x3f		/* 0x01000000 */

#define msk_X0		0x00000002
#define msk_X1		0x00000001
#define msk_Y0		0x00000008
#define msk_Y1		0x00000004
#define msk_A0		0x00000040
#define msk_B0		0x00000200
#define msk_A2		0x00000010
#define msk_B2		0x00000080
#define msk_A1		0x00000020
#define msk_B1		0x00000100
#define msk_RNM(r)	(1 << ((r)+10))
#define msk_SR		0x00040000
#define msk_OMR		0x00080000
#define msk_SP		0x00100000
#define msk_SSH		0x00200000	/* do not read! */
#define msk_SSL		0x00400000	/* do not read! */
#define msk_LA		0x00800000
#define msk_LC		0x01000000

#define O_moviR0	0x60F400	/* move #xxxx,r0   */
#define O_moviX0	0x44F400	/* move #xxxxxx,x0 */

#define O_movR0p	0x08D87C	/* move S:(r0)+,X:OGDB */
#define O_movAp		0x08F07C	/* move abs,X:OGDB     */
/* OR with  P=0x00  X=0x40  Y=0x80 */

#define O_jmplong	0x0AF080	/* jmp >$xxxx   */

/* -------------------------------------------------------------------- */

#define OnCEwdat16(r,v,a)	OnCEwdat((r),((long)(v)<<8),(a))

#define ReadSCR()		OnCErdat(O_OSCR|O_read)
#define WriteSCR(v)		OnCEwdat(O_OSCR|O_write,v,0)
#define ReadPDB()		OnCErdat(O_OPDBR|O_read)
#define WritePDB(v)		OnCEwdat(O_OPDBR|O_write,v,0)
#define WritePDBAck(v)		OnCEwdat(O_OPDBR|O_write,v,1)
#define WritePDBgo(v)		OnCEwdat(O_OPDBR|O_write|O_go,v,0)
#define WritePDBgoAck(v)	OnCEwdat(O_OPDBR|O_write|O_go,v,1)
#define Execute1i(v)		WritePDBgoAck(v)
#define Execute2i1(v)		WritePDBAck(v)
#define Execute2i2(v)		WritePDBgoAck(v)
#define WritePDBex(v)		OnCEwdat(O_OPDBR|O_write|O_ex,v,0)
#define WritePDBgoex(v)		OnCEwdat(O_OPDBR|O_write|O_go|O_ex,v,0)
#define ReadGDB()		OnCErdat(O_OGDBR|O_read)
#define ReadGDBgo()		OnCErdat(O_OGDBR|O_read|O_go)
#define ReadGDBex()		OnCErdat(O_OGDBR|O_read|O_ex)
#define NOPgo()			send_command(O_noreg|O_go)

#define begin_debug(r)		begin_debug_run(r,0)
#define begin_debug_res(r)	begin_debug_run(r,1)
#define end_debug(r)		end_debug_go(r,0)
#define end_debug_cont(r)	end_debug_go(r,0)

#define loadX(a,n)		OnCEwmem('x',a,n,DSPBUF)
#define loadY(a,n)		OnCEwmem('y',a,n,DSPBUF)
#define loadP(a,n)		OnCEwmem('p',a,n,DSPBUF)
#define fillX(a,n,v)		OnCEfillmem('x',a,n,v)
#define fillY(a,n,v)		OnCEfillmem('y',a,n,v)
#define fillP(a,n,v)		OnCEfillmem('p',a,n,v)
#define clearX(a,n)		OnCEfillmem('x',a,n,0)
#define clearY(a,n)		OnCEfillmem('y',a,n,0)
#define clearP(a,n)		OnCEfillmem('p',a,n,0)

/* ==================================================================== */

extern u_long bk_hi,bk_lo,bk_cnt;
extern int isbk;

int OnCEreq(void);
int OnCErunmode(void);
int enterOnCE(int reset);
int send_command(register int n);
int send_data24(u_long v,int ack);
long get_data24(int ack);
long OnCErdat(u_char reg);
int OnCEwdat(int reg,u_long v,int ack);
int begin_debug_run(OnCEregs *o,int reset);
int end_debug_go(OnCEregs *o,u_long a);	/* if a==0 no jump */
int OnCEtrace(OnCEregs *o,u_long n,u_long a);
int OnCEbrkgo(OnCEregs *o);
int OnCEdebgo(OnCEregs *o);
u_long OnCErreg(int r);		/* r = register NUM */
int OnCEwreg(int r,u_long v);	/* r = register NUM */
int OnCEgetallreg(DSPregs *r,u_long mask);
int OnCErmem(char space,u_long a,int n,u_long *buf);
int OnCEwmem(char space,u_long a,int n,u_long *buf);
int OnCEfillmem(char space,u_long a,int n,u_long v);

/* ==================================================================== */

#endif
#endif
