
/* ==================================================================== */
/*			DTC2.h	08/01/96				*/
/* ==================================================================== */
 
#ifndef _LINUX_DTC2_H
#define _LINUX_DTC2_H

#ifdef  CONFIG_DTC2
/*
#if	LOC_DTC2_QRAM==1
#define	DTC2_QRAM	0xcc000
#elif	LOC_DTC2_QRAM==2
*/
#define	DTC2_QRAM	0xd0000
/*
#elif	LOC_DTC2_QRAM==3
#define	DTC2_QRAM	0xee000
#else
#error	No QRAM defined, please make config
#endif
*/

#undef PACKED
#if __GNUC__ >= 2
# define PACKED 	__attribute__((packed))
#else
# error dont know how to pack structures
#endif

/* ==================================================================== */
/*	PLD6A								*/
/* ==================================================================== */

#define IDECS0		0x1e8
#define IDECS1		0x3ec

#define iXINIT		0x160
#define iXDONE		0x164
#define iXMODE		0x168
#define iLOCAL		0x16c

/* ==================================================================== */
/*	ALS259 bits, W/O, 0160-0167					*/
/* ==================================================================== */

#define oTRESET		0x160
#define oTANALY		0x161
#define oECF2		0x162
#define oECF3		0x163
#define oXMODE		0x164
#define oDRES		0x165
#define oDMODE		0x166
#define oXPROG		0x167

/* ==================================================================== */
/*	DTC/2 XC4003 FPGA - base version				*/
/*
	Addr
	0150-0158	R/W	C012 (decode)
	0178		W	Control lines
				b0	IRQ9 enable (capture)
				b1	IRQ5 enable (C012 IRQ)
				b2	EVREN EvReq enable
				b3	Reset 401 registers
				b4	NMI enable (capture)
				b5	TFIRQ enable to DSP (capture)
				b6	SPECIAL FEATURE enable
				b7	capture enable
			R	capture data bus
	0179		R	capture addr/flags
				b0-b2	addr bus b0-b2
				b3-b4	capture port
					00	no port
					01	0378
					10	03bc
					11	0100
				b5	IRQ9 enable echo
				b6	capture R(1)/W(0)
				b7	doble capture error
	0301-0307(odd)	R	M401 status bits
				b6	0=ready to write
				b7	0=ready to read
	0300-0306(even)	R/W	M401 data register (=QRAM:030x)
	0301-0307(odd)	W	M401 control register (=QRAM:030x)
	0340-0347	R/W	56002 host port
	0348		W	b0	1=set DR,clr ACK
					0=clr DR,clr ACK
	0348		R	OnCE status
				b0	DSI/OS0
				b1	DSCK/OS1
				b2	ACK
				b3	Q5ACT
				b4	DSO
	0349		R/W	OnCE data register
	034A		W	start OnCE send/receive
	034B		W	b0	EvReq enable(1) if EVREN=1
	034B		R	T-status
				b0	EvAck
				b1	EvReq
				b2	IINT
				b3	OINT

   unused pins on FPGA:
	5	12	13	21	24	30	33
	36	37	45	47	48	59	65
	72
 */
/* ==================================================================== */

#define FPGA		0x178

#define M401		0x300

#define DSP56		0x340
#define ONCE		(DSP56+8)
#define C012		0x150

/* ==================================================================== */

/*
 * These are the dsp_table[].flags flags...
 */
#define DSP_EXIST 		0x0001
#define DSP_BUSY  		0x0002	/* Is the board in use ? */

/*
 * Defines for easier access to the link_table.
 */
#define DSP_F(minor)		dsp_flags[(int) minor]

typedef int DSPMuLaw;
typedef int DSPFix8;
typedef int DSPFix16;
typedef int DSPFix24;
typedef struct _DSPFix48 {
    int high24;		      /* High order 24 bits, right justified */
    int low24;		      /* Low order 24 bits, right justified */
} DSPFix48;

typedef DSPFix16 DSPAddress;
typedef DSPFix24 DSPDatum;
typedef DSPFix48 DSPLongDatum;
typedef DSPFix48 DSPTimeStamp;

typedef int DSP_BOOL;
#define DSP_TRUE 1
#define DSP_FALSE 0
#define DSP_NOT_SET 2
#define DSP_MAYBE (-2)		/* TRUE and FALSE defined in nextstd.h */
#define DSP_UNKNOWN (-1)	/* like DSP_{MAYBE,NOT_SET} for adresses */

/* ==================================================================== */

/* mpu-401 command and message summary */

#define     NOTE_OFF            0x80    /* MIDI voice messages - channel 1 */
#define     NOTE_ON             0x90
#define     POLY_AFTERTOUCH     0xA0
#define     MODE_MESSAGE        0xB0
#define     PROGRAM_CHANGE      0xC0
#define     AFTERTOUCH          0xD0
#define     PITCH_WHEEL         0xE0
#define     ALL_NOTES_OFF       0x7B    /* channel mode message */

/* ----------------------------------------------------------------- */

#define HI_ICR		(DSP56)
#define HI_CVR		(DSP56+1)
#define HI_ISR		(DSP56+2)
#define HI_IVR		(DSP56+3)
#define HI_TXH 		(DSP56+5)
#define HI_TXM 		(DSP56+6)
#define HI_TXL 		(DSP56+7)
#define HI_RXH 		HI_TXH
#define HI_RXM          HI_TXM
#define HI_RXL          HI_TXL

#define MS_X	0
#define MS_Y	1
#define MS_P	2
#define MS_R	3
#define MS_E	4

/* Host vectors are 01-63, Host commands 18-30 and 32-63 */

#define HOSTCMD(n)	((n)+0x12)
#define HC_Peek		0x12
#define HC_Poke		0x13
#define HC_Fill		0x14

static __inline__ void _TXwrite (DSPFix24 val)
{
#ifdef __KERNEL__
  __asm__ __volatile__ ("movl %0,%%eax\n\tmovw %1,%%dx"
  	"\n\trorl $16,%%eax"
  	"\n\tcli"
  	"\n\toutb %%al,%%dx"
  	"\n\troll $8,%%eax"
  	"\n\tincw %%dx"
  	"\n\toutb %%al,%%dx"
  	"\n\troll $8,%%eax"
  	"\n\tincw %%dx"
  	"\n\toutb %%al,%%dx"
  	"\n\tsti"
  	:
	: "g"(val), "n"(HI_TXH)
	: "eax", "edx", "memory");
#else
  __asm__ __volatile__ ("movl %0,%%eax\n\tmovw %1,%%dx"
  	"\n\trorl $16,%%eax"
  	"\n\toutb %%al,%%dx"
  	"\n\troll $8,%%eax"
  	"\n\tincw %%dx"
  	"\n\toutb %%al,%%dx"
  	"\n\troll $8,%%eax"
  	"\n\tincw %%dx"
  	"\n\toutb %%al,%%dx"
  	:
	: "g"(val), "n"(HI_TXH)
	: "eax", "edx", "memory");
#endif
}


static __inline__ DSPFix24 _RXread (void)
{
 register int val;

#ifdef __KERNEL__
  __asm__ __volatile__ ("xorl %%eax,%%eax\n\tmovw %1,%%dx"
  	"\n\tcli"
	"\n\tinb  %%dx,%%al"
	"\n\tshll $8,%%eax"
	"\n\tincw %%dx"
	"\n\tinb  %%dx,%%al"
	"\n\tshll $8,%%eax"
	"\n\tincw %%dx"
	"\n\tinb  %%dx,%%al"
  	"\n\tsti"
	"\n\tmovl %%eax,%0"
	: "=g"(val)
	: "n"(HI_RXH)
  	: "eax", "edx", "memory");
#else
  __asm__ __volatile__ ("xorl %%eax,%%eax\n\tmovw %1,%%dx"
	"\n\tinb  %%dx,%%al"
	"\n\tshll $8,%%eax"
	"\n\tincw %%dx"
	"\n\tinb  %%dx,%%al"
	"\n\tshll $8,%%eax"
	"\n\tincw %%dx"
	"\n\tinb  %%dx,%%al"
	"\n\tmovl %%eax,%0"
	: "=g"(val)
	: "n"(HI_RXH)
  	: "eax", "edx", "memory");
#endif
  return val;	
}

/* -------------------------------------------------------------------- */

#define	QR_PAGE0		(DTC2_QRAM+0x000)
#define	QR_PAGE1		(DTC2_QRAM+0x100)
#define	QR_PAGE2		(DTC2_QRAM+0x200)
#define	QR_PAGE3		(DTC2_QRAM+0x300)

typedef struct {
	u_char	data;
	u_char	stat;
 } Q401Port;
 
typedef struct {
	Q401Port	Q401[4]		PACKED;	/* c300-7 */
	u_long		ident		PACKED;	/* c308-b */
	char		xx1[44]		PACKED;	/* dummy  */
	Q401Port	X401[2]		PACKED;	/* c338-b */
	u_char		xx2[4]		PACKED;
	u_char		xc340		PACKED;
	u_char		QCVR		PACKED;	/* c341: aux CVR */
	u_char		xc342		PACKED;
	u_char		YSTAT		PACKED;	/* c343: YM3812 status */
	u_char		IRQP		PACKED;	/* c344: IRQ PAL */
	u_char		Qe003		PACKED;
	u_char		QCAPT1		PACKED;	/* c346 */
	u_char		QCAPT2		PACKED;
	u_long		QTIML		PACKED;	/* c348 */
	u_long		QTIMH		PACKED;	/* c34c */
	u_char		UARTIN[4]	PACKED;
	u_short		DAC		PACKED;	/* c354 */
	u_short		ADC		PACKED;	/* c356 */
	u_char		xx3[0x28]	PACKED;	/* c358 */
	u_char		MIDIXT[0x40]	PACKED;	/* c380 */
	u_char		xx4[0x40]	PACKED;	/* c3c0 */
 } Qrdx;


/* -------------------------------------------------------------------- */

typedef struct {
  u_char db;	/* from read 0x179 */
  union {
    u_char a;
    struct {
      u_int ab : 3;
      u_int ch : 2;
      u_int en : 1;
      u_int rw : 1;
      u_int err: 1;
     } b;
   } eb;
 } CaptElem;

/* -------------------------------------------------------------------- */
/* IOCTLS */

#define DTC2_READIDENT		0x100
#define DTC2_READTIMER		0x101
#define DTC2_RESETDSP		0x200
#define DTC2_READDSP		0x201
#define DTC2_WRITEDSP		0x202

/* -------------------------------------------------------------------- */

#ifndef min
#define min(a,b) \
	({int _a=(a), _b=(b); _a < _b ? _a : _b; })
#endif
#ifndef max
#define max(a,b) \
	({int _a=(a), _b=(b); _a > _b ? _a : _b; })
#endif

#endif	CONFIG_DTC2

#endif	_LINUX_DTC2_H

