
/************************************************************************
*									*
*	Device Driver for DSP56002/DTC2                                 *
*									*
*************************************************************************/

#ifdef MODULE
# include <linux/config.h>
# include <linux/module.h>
#endif
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/major.h>

#include <asm/io.h>
#include <asm/segment.h>

#include <linux/dtc2.h>
#include "OnCE.h"

#define CARD_NAME	"dtc2"
#define FORCE_CFG

#if 1	/* linux 2.1 */
#include <linux/string.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#define dtc2_memcpy_fromio	memcpy_fromio
#define dtc2_memcpy_toio	memcpy_toio
#define dtc2_memcpy_fromfs	copy_from_user
#define dtc2_memcpy_tofs	copy_to_user
#else
#define dtc2_memcpy_fromio	memcpy_tofs
#define dtc2_memcpy_toio	memcpy_fromfs
#define dtc2_memcpy_fromfs	memcpy_fromfs
#define dtc2_memcpy_tofs	memcpy_tofs
#endif

#ifdef DSP_DEBUG
#define DEB(x) x
#define PRINTK(f, a...)	printk(f, ##a)
#define NO_DEB(x) /* */
#else
#define DEB(x) /* */
#define PRINTK(f, a...)
#define NO_DEB(x) x
#endif

#ifdef DSP_DEBUG_MORE
#define DEB_MORE(x) x
#define PRINTKM(f, a...)	printk(f, ##a)
#else
#define DEB_MORE(x) /* */
#define PRINTKM(f, a...)
#endif

static int in_use = 0;			/* Total # of open devices */

static int use_schedule = 0;            /* don't use schedule() in init() */
static int DSPexists = 0;
static int DSPowner = 0;
static int OnCEmode = 0;
/*static int preinit = 0;*/
static int HWxmode = 0;
static int FPGAerr,FPGAdone,FPGAmode;
static unsigned char r178;

/* map DSP exchange area into QRAM */
static Qrdx *QRDX = (Qrdx *)QR_PAGE3;

#define TCNT1	20000
#define TCNT2	10000

extern int fpgaconfTAB(void);

static int dsp_flags[16] = { 0, 0 };

static char csbuf[1024];

/****************************************************************************/


void d_delay( int il )
{
	if ( use_schedule && !il )
	{
		DEB(printk("dsp_delay: Using schedule()\n");)
		current->state = TASK_INTERRUPTIBLE;
		current->timeout = jiffies + 10;   /* 100ms */
		schedule();
	}
	else
	{	/* only for detection in xxx_init() */
		volatile int i;
		DEB(printk("dsp_delay: Using idle loop\n");)
		for (i = 0; i < 200000; i++);
	}
}


/* -------------------------------------------------------------------- */
/*
 * /dev/dtcstatus -device (minor 6)
 */
static char    *status_buf = NULL;
static int      status_len, status_ptr;
static int      status_busy = 0;

static int
put_status (char *s)
{
  int             l = strlen (s);

  if (status_len + l >= 4000)
    return 0;

  memcpy (&status_buf[status_len], s, l);
  status_len += l;

  return 1;
}

static int
put_status_int (unsigned int val, int radix, int pad)
{
  int             l, v;

  static char     hx[] = "0123456789abcdef";
  char            buf[11];

  if (!val)
    return put_status ("0");

  l = 0;
  buf[10] = 0;

  while (val || pad)
    {
      v = val % radix;
      val = val / radix;

      buf[9 - l] = hx[v];
      l++;
      if (pad) pad--;
    }

  if (status_len + l >= 4000) return 0;

  memcpy (&status_buf[status_len], &buf[10 - l], l);
  status_len += l;

  return 1;
}

static void
init_status (void)
{
  static char ssep[] =
    "--------------------------------------------------------------------\n";
  status_ptr = 0;

  if (!put_status (ssep)) return;
  if (!put_status ("DTC2 ["__DATE__" "__TIME__"]  ID=")) return;
  if (!put_status_int(readl(&QRDX->ident),16,6)) return;
  if (!put_status("\n")) return;
  if (!put_status (csbuf)) return;
  put_status ("FPGA:");
  if (!put_status_int(inb(FPGA),16,2)) return;
  if (!put_status_int(inb(FPGA+1),16,2)) return;
  put_status ("  QCVR:");
  if (!put_status_int(readb(&QRDX->QCVR),16,2)) return;
  put_status ("  YSTAT:");
  if (!put_status_int(readb(&QRDX->YSTAT),16,2)) return;
  put_status ("  IRQP:");
  if (!put_status_int(readb(&QRDX->IRQP),16,2)) return;
  put_status ("  QTIM:");
  if (!put_status_int(readl(&QRDX->QTIMH),16,8)) return;
  if (!put_status_int(readl(&QRDX->QTIML),16,8)) return;
  if (!put_status("\n")) return;
  if (!put_status (ssep)) return;
}


static ssize_t read_status(struct file *filp, char *buf, size_t count,
		loff_t *ppos)
{
  /*
   * Return at most 'count' bytes from the status_buf.
   */
  int             l, c;

  l = count;
  c = status_len - status_ptr;

  if (l > c) l = c;
  if (l <= 0) return 0;

  dtc2_memcpy_tofs (&(buf)[0], &status_buf[status_ptr], l);
  status_ptr += l;

  return l;
}


/* -------------------------------------------------------------------- */


static int DTC2present(void)
{
  int err;

	err=1;
/* reset XMODE */
	outb(0,oXMODE);
	HWxmode = inb(iXMODE)&1;	/* 1 if inv(PC), 0 if PROM */

/* toggle/test XMODE */
	outb(1,oXMODE);
	if ((inb(iXMODE)&1)^HWxmode) err=0;
	outb(0,oXMODE);
	if ((inb(iXMODE)&1)^HWxmode) err=0;
	return(!err);
}


/****************************************************************************
 *
 */
 
static ssize_t read_dmem(struct file *file, char *buf, size_t count,
		loff_t *ppos)
{
	unsigned long p = *ppos & 0x1ff;
	int c1, c2, read;

	if ((count < 0) || (count > 512))
		return -EINVAL;
	c1 = min (count, (512 - p));
	c2 = count + p;
	read = 0;
	if (c1) dtc2_memcpy_fromio(buf, (void *) (QR_PAGE0 + p), c1);
	if (c2 > 512) dtc2_memcpy_fromio(buf+c1, (void *) QR_PAGE0, (c2 & 0x1ff));
	read += count;
	*ppos = (*ppos + read) & 0x1ff;
	return read;
}


static ssize_t write_dmem(struct file *file, const char *buf, size_t count,
		loff_t *ppos)
{
	unsigned long p = *ppos & 0x1ff;
	int c1, c2, written;

	if ((count < 0) || (count > 512))
		return -EINVAL;
	c1 = min (count, (512 - p));
	c2 = count + p;
	written = 0;
	if (c1) dtc2_memcpy_toio((void *) (QR_PAGE0 + p), buf, c1);
	if (c2 > 512) dtc2_memcpy_toio((void *) QR_PAGE0, buf+c1, (c2 & 0x1ff));
	written += count;
	*ppos = (*ppos + written) & 0x1ff;
	return count;
}


/****************************************************************************/


static inline void TXwrite_b(DSPFix24 val)	/* blocking */
{
 unsigned count = TCNT1;

  while ((inb(HI_ISR)&2)==0) {
    if ((--count)==0) { count=TCNT1; schedule(); }
  }
  _TXwrite(val);
}

static inline int TXwrite_n(long val)	/* nonblocking */
{
  if ((inb(HI_ISR)&2)==0) return -1;
  _TXwrite((DSPFix24)val);
  return 0;
}


static inline DSPFix24 RXread_b(void)	/* blocking */
{
 unsigned count = TCNT1;

  while ((inb(HI_ISR)&1)==0) {
    if ((--count)==0) { count=TCNT1; schedule(); }
  }
  return(_RXread());
}

static inline long RXread_n(int *err)	/* nonblocking */
{
  if ((inb(HI_ISR)&1)==0) { *err=-1; return 0; }
  *err = 0;
  return ((long)_RXread());
}


static inline int DSPcommand(unsigned char c)
{
 unsigned count=TCNT1,ntest;

  while (inb(HI_CVR)&0x80) {
    if ((--count)==0) { count=TCNT1; schedule(); }
  }
  outb((c|0x80), HI_CVR);
  count = TCNT1; ntest=5;
  while ((inb(HI_CVR)&0x80)&&ntest) {
    if ((--count)==0) { count=TCNT1; ntest--; schedule(); }
  }
  return(ntest==0? -1:0);
}


long DSPreset (int mode)
{
	outb(0,oDRES);
	outb(((~mode>>2)&1),oECF2);
	outb(((mode>>1)&1),oECF3);
	outb((mode&1),oDMODE);
	d_delay(1);
	outb(1,oDRES);
	outb(0,ONCE);
	d_delay(0);
	return((long)_RXread());
}


/****************************************************************************
 *
 */
static ssize_t dsp_read(struct file *filp, char *buf, size_t count,
		loff_t *ppos)
{
	static long v;
	int l, n;

/*	PRINTKM("DSP(%d): reading %d bytes (polling mode).\n", minor, count); */

	if ( count < 0)
	{
/*		PRINTK("DSP: invalid argument for reading: count = %d.\n", count); */
		return -EINVAL;
	}
	if (MINOR(filp->f_dentry->d_inode->i_rdev)==0) {
		l = 0;
		while (count) {
			v = RXread_b();
			n = (count<4? count:4);
			dtc2_memcpy_tofs (buf, &v, n);
			count-=n; buf+=n; l+=n;
		}
		*ppos += l;
		return l;
	}
	return -ENODEV;
}


/****************************************************************************
 *
 */
static ssize_t dsp_write(struct file *filp, const char *buf, size_t count,
		loff_t *ppos)
{
	static long v;
	int l;

/*	PRINTKM("DSP(%d): writing %d bytes (polling mode).\n", minor, count); */

	if ( count < 0)
	{
/*		PRINTK("DSP: invalid argument for reading: count = %d.\n", count); */
		return -EINVAL;
	}
	if (MINOR(filp->f_dentry->d_inode->i_rdev)==0) {
		l = 0;
		while (count > 3) {
			dtc2_memcpy_fromfs(&v, buf, 4);
			TXwrite_b(v);
			count-=4; buf+=4; l+=4;
		}
		*ppos += l;
		return l;
	}
	return -ENODEV;
}


/****************************************************************************
 *
 */
static loff_t dsp_lseek(struct file * file, loff_t offset, int origin)
{
	switch (origin) {
		case 0:
			file->f_pos = offset;
			return file->f_pos;
		case 1:
			file->f_pos += offset;
			return file->f_pos;
		default:
			return -EINVAL;
	}
	if (file->f_pos < 0)
		return 0;
	return file->f_pos;
}


/****************************************************************************
 *
 */
static int dsp_release(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);

	if (minor > 15) return 0;

	DSP_F(minor) &= ~DSP_BUSY;

	if (minor==6) {
		if (status_buf)	vfree (status_buf);
		status_buf = NULL;
		status_busy = 0;
	}

#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif

	PRINTK("DSP(%d) released.\n", minor);
	return 0;
}


/****************************************************************************
 *
 */
static int dsp_ioctl(struct inode *inode, struct file *file,
		    unsigned int cmd, unsigned long arg)
{
/*	const unsigned int minor = MINOR(inode->i_rdev); */
	int result = -1;

/*	PRINTK("DSP(%d) ioctl, cmd: 0x%08x, arg: 0x%08x.\n", minor, cmd, (int) arg); */

	if (!DSPexists) return -ENODEV;
	
/*	if (minor > 15) */ return -EINVAL;

	switch (cmd) {
		case DTC2_READIDENT:
			result = readl(&QRDX->ident);
			break;
		case DTC2_READTIMER:
			result = readl(&QRDX->QTIML);
			break;
		case DTC2_RESETDSP:
			result = DSPreset(arg&7);
			break;
		case DTC2_READDSP:
		{
			unsigned long v;
			v = RXread_n(&result);
			if (arg) dtc2_memcpy_tofs((void *)arg, &v, 4);
			break;
		}
		case DTC2_WRITEDSP:
			result = TXwrite_n(arg);
			break;
	}	
	return result;
}


/*****************************************************************************/
/* 2.1.60:
 * struct file_operations {
 *	loff_t (*llseek) (struct file *, loff_t, int);
 *	ssize_t (*read) (struct file *, char *, size_t, loff_t *);
 *	ssize_t (*write) (struct file *, const char *, size_t, loff_t *);
 *	int (*readdir) (struct file *, void *, filldir_t);
 *	unsigned int (*poll) (struct file *, poll_table *);
 *	int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
 *	int (*mmap) (struct file *, struct vm_area_struct *);
 *	int (*open) (struct inode *, struct file *);
 *	int (*release) (struct inode *, struct file *);
 *	int (*fsync) (struct file *, struct dentry *);
 *	int (*fasync) (struct file *, int);
 *	int (*check_media_change) (kdev_t dev);
 *	int (*revalidate) (kdev_t dev);
 *	int (*lock) (struct file *, int, struct file_lock *);
 * };
 */

static struct file_operations mem_fops = {
	NULL,		/* llseek */
	read_dmem,
	write_dmem,
	NULL,		/* readdir */
	NULL,		/* poll */
	NULL,		/* ioctl */
	NULL,		/* mmap */
	NULL,		/* open */
	dsp_release,
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};

static struct file_operations stat_fops = {
	NULL,		/* llseek */
	read_status,
	NULL,		/* write */
	NULL,		/* readdir */
	NULL,		/* poll */
	NULL,		/* ioctl */
	NULL,		/* mmap */
	NULL,		/* open */
	dsp_release,
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};

static struct file_operations dsp_fops = {
	dsp_lseek,
	dsp_read,
	dsp_write,
	NULL,		/* readdir */
	NULL,		/* poll */
	dsp_ioctl,
	NULL,		/* mmap */
	NULL,		/* open */
	dsp_release,
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};


static int dsp_open(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);

	if (minor > 15) {
		filp->f_op = &mem_fops;
		return 0;
	}

	if (DSP_F(minor) & DSP_BUSY)
	{
		PRINTK("DSP(%d) not opened, device busy.\n", minor);
		return -EBUSY;
	}
	DSP_F(minor) |= DSP_BUSY;

	if (minor==6) {
		filp->f_op = &stat_fops;
		if ((status_buf = (char *) vmalloc (4000)) == NULL) return -(EIO);
		status_len = status_ptr = 0;
		init_status ();
	}
	else
		filp->f_op = &dsp_fops;

#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif

	PRINTK( "DSP(%d) opened.\n", minor);
	return 0;

}


/****************************************************************************/

static struct file_operations dtc2_fops = {
	NULL,		/* llseek */
	NULL,		/* read */
	NULL,		/* write */
	NULL,		/* readdir */
	NULL,		/* poll */
	NULL,		/* ioctl */
	NULL,		/* mmap */
	dsp_open,
	NULL,		/* release */
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};


/****************************************************************************
 *
 * This function initializes the driver. It tries to detect the hardware
 * and sets up all relevant data-structures.
 *
 ****************************************************************************/


static int DTCinitp2(void)
{
  u_int k;
  char *p;
  int retry,retcode,qid;

	p = csbuf;  
	p += sprintf(p,"DTC2 init ["__DATE__" "__TIME__"]\n");
	DSPexists = 0;
	
	if (!DTC2present()) {
		printk("DTC2 not found\n");
		retcode=1; goto quit;
	}
	FPGAerr  = inb(iXINIT)&1;
	FPGAdone = inb(iXDONE)&1;
	FPGAmode = inb(iXMODE)&1;
	p += sprintf(p,"FPGA mode=%d err=%d done=%d - ",FPGAmode,FPGAerr,FPGAdone);

	k = inb(iXDONE)&1;
	if (k) printk("FPGA already configured\n");
	 else {
		printk("FPGA not configured\n");
#ifdef FORCE_CFG
	  }
#endif
		outb (1, oXMODE);
		if (fpgaconfTAB()) {
			retcode=1; goto quit;
		}
#ifndef FORCE_CFG
	  }
#endif
	outb(0, oXMODE);
	outb(8, FPGA);		/* reset MPU401 */
/*
   [0]IRQ9EN=0  [1]IRQ5EN=0  [2]EVREN=0  [3]RESET401=pulse  [4]NMIEN=0
   [5]TFIRQ=0   [6]CTRL6=1   [7]CAPTEN=0
*/
	r178 = 0;
	outb(r178, FPGA);

	use_schedule = 1;
	DSPowner = 0;
	OnCEmode = 0;
	retry = 0;
again:
	retcode = 0;
	qid = readl(&(QRDX->ident));
	if (qid == 0x435444) {
		p += sprintf(p,"DTC2 initialized");
		k = readl(&(QRDX->QTIML));
		d_delay(1);
		if (k != readl(&(QRDX->QTIML))) {
			p += sprintf(p," and running, t=%d sec",k/10000);
			DSPexists = 1;
		}
		else if ((inb(OnCEstat)&7)==4) {
			p += sprintf(p," in debug mode");
			DSPexists = 1;
			OnCEmode=1;
		}
		p += sprintf(p,"\n");
	}
	else if (qid == 0x444e45) {
		p += sprintf(p,"DTC2 previously stopped\n");
	}
	if (!DSPexists && !retry) {
		p += sprintf(p,"...resetting...");
		writel(0xffffff,&(QRDX->ident));
		k=DSPreset(1);
		p += sprintf(p,"says %06x\n",k);
		retry=1;
		d_delay(0);
		goto again;
	}
	if (!DSPexists) retcode=1;

	for (k=0; k<64; k++)
		writeb(k,&(QRDX->MIDIXT[k]));
quit:
	return retcode;
}


#ifndef MODULE

long dtc2_init(long kmem_start)
{
	
	if ( register_chrdev( DSP56_MAJOR, "dsp56", &dtc2_fops ) )
	{
		printk("dsp_init: unable to get major %d for 56002 interface.\n",
			DSP56_MAJOR );
		return kmem_start;
	}

	DTCinitp2();

	return kmem_start;

}

#endif

/****************************************************************************/

#ifdef MODULE

int init_module(void) {

	if (sizeof(Qrdx)!=256) return 1;

	if (DTCinitp2()) return(-ENODEV);

	if ( register_chrdev( DSP56_MAJOR, "dsp56", &dtc2_fops ) )
	{
	  printk("dsp_init: unable to get major %d for 56002 interface.\n",
		DSP56_MAJOR );
	  return -EIO;
	}

	request_region(oECF2,14,CARD_NAME);
	request_region(FPGA,4,CARD_NAME);
	request_region(M401,8,CARD_NAME);
	request_region(DSP56,12,CARD_NAME);

	status_len = status_ptr = 0;

	in_use = 1;
	return 0;
}


void cleanup_module( void) {

  if (MOD_IN_USE)
	printk("dtc2: device busy, remove delayed\n");

  if (unregister_chrdev(DSP56_MAJOR, "dsp56") != 0) {
	printk("cleanup_module failed\n");
  } else {
	writel(0x444e45,&(QRDX->ident));
	outb(0,oDRES);
	outb(0,oDMODE);
	outb(1,oTRESET);
	release_region(oECF2,14);
	release_region(FPGA,4);
	release_region(M401,8);
	release_region(DSP56,12);

	printk("DTC2 cleanup_module succeeded\n");
	in_use = 0;
  }    
}

#endif	/* MODULE */

/****************************************************************************/
