
/* ================================================================= */
/*                                                                   */
/*		LOW-LEVEL FPGA interface                             */
/*                                                                   */
/* ================================================================= */

#ifdef __KERNEL__
# define PRINTK(a...)	printk(a)
#else
# include <stdio.h>
# define PRINTK(a...)	fprintf(stderr, ##a)
#endif
#define __NO_VERSION__
#include <asm/io.h>
#include <asm/segment.h>

#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>

#include <linux/dtc2.h>

#include "fpgabits.i"

static void nDelay(void)
{
	__asm__ __volatile__ (
		"\noutb %al,$0x80"
		"\noutb %al,$0x80"
		"\noutb %al,$0x80"
		"\noutb %al,$0x80" );
}

static void pDelay(int j)
{
	current->state = TASK_INTERRUPTIBLE;
	current->timeout = jiffies + j;   /* j*10ms */
	schedule();
}

/* ----------------------------------------------------------------- */


int fpgaconfTAB(void)
{
 int t,err;
 unsigned char *b;
 u_int i,ofs;
 u_long len;

/* pulse Program low */
  i=0;
  outb(1,oXPROG);	/* inv */
  pDelay(20);
  outb(0,oXPROG);	/* inv */
/* wait for INIT high */
  while ((inb(iXINIT)&1)==0) i++;
  printk("FPGA is reset (t=%04x)\n",i);

  b=FPGABITS;
  len=sizeof(FPGABITS);
  err=0;
  ofs=0;
  while (len--) {
retry:
    t=80000; while (((inb(FPGA)&0x80)==0) && (--t));
    if (t==0) { err=4; break; }
    if ((inb(FPGA)&0x80)==0) goto retry;
    outb(*b++,FPGA);
    if (ofs==21) { nDelay(); nDelay(); } nDelay();	/* kludge */
    if ((inb(iXINIT)&1)==0) { err=1; break; }
    if (inb(iXDONE)&1) { err=2; break; }
    ofs++;
   }
  printk("%d bytes written\n",ofs);
  if (!err) {
    outb(0xff,FPGA);
    pDelay(5);
    if ((inb(iXDONE)&1)==0) err=3;
   }
  switch (err) {
    case 1: printk("INIT/ERR low at %04x\n",ofs);
	    return (1);
    case 2: printk("early DONE at %04x\n",ofs);
	    return (1);
    case 3: printk("not yet DONE\n");
	    return (1);
    case 4: printk("BUSY timeout\n");
	    return (1);
   }

  if (!err) {
    pDelay(5);
/* test echo of CTRL0 bit */
    outb(1,FPGA);
    if ((inb(FPGA+1)&0x20)==0) err=1;
    outb(0,FPGA);
    if ((inb(FPGA+1)&0x20)!=0) err=1;
   }
  outb(1,oXMODE);	/* set FPGA mode always to 1 */
  if (err) return (1);

  i = inb(FPGA);
  printk("FPGA initialized, version is %02x:%02x\n",i,inb(FPGA+1)); 

  return(0);
}

