
/* ==================================================================== */
/*			DTC3.h	08/21/98				*/
/*			internal definitions				*/
/* ==================================================================== */
 
#ifndef _LINUX_DTC3I_H
#define _LINUX_DTC3I_H

#include "dtc3.h"
#include <asm/uaccess.h>

#define FPGA		0x158		/* hardwired, depends on PAL */
#define FPGA_PGM	0x8158

#define DONE		0x40
#define nERROR		0x20
#define INIT		0x20
#define DOUT		0x10

#define C012A		0x150		/* 150..153 */
#define C012B		0x550		/* 550..553 */

#define FPGAREPR	0x154		/* write 0x5a */
#define SXINPORT	0x154		/* read SXIN0-3 = bits 0-3 */

#define PBDMRPORT	0x155		/* bits 3,5 */
#define PBDMWPORT	0x156		/* bits 0-3 */

/* -------------------------------------------------------------------- */
#define CTRLPORT	0x157		/* R/W, default=0x10 */

/* READ DRAM start address
	0.000=1.000	d0000		   0x10 = hardcoded default
	0.001=1.001	d2000
	0.010=1.010	d4000
	0.011=1.011	d6000
	0.100		c8000
	0.101		ca000
	0.110		cc000
	0.111		ce000
	1.100		d8000
	1.101		da000
	1.110		dc000
	1.111		de000
*/
#define MASK_DRAM	0x1f		/* shared RAM relocation (v0=0x1e) */

/* same bits: WRITE */
#define MASK_EXOUT	0x0f
#define MASK_IRQON	0x10

#define MASK_A12	0x20		/* 68k mem bank select */
#define MASK_MIDE	0x40		/* MIDI enable, active=0 */
#define MASK_XC12	0x80		/* exchange C012s if 1 */

/* -------------------------------------------------------------------- */
#define HTIMER		0x555

#define IRQSTAT		0x556
#define IQR_WSTAT	0x0f		/* 4 MIDI ports write stat, 1=empty */
#define IQR_HWINT	0x10		/* physical irq line */
#define IQR_HIRQ	0x20
#define IQR_TIRQ	0x40		/* timer irq active */
#define IQR_MIRQ	0x80		/* MIDI ready irq active */

#define IRQCTRL		0x557
#define IQW_FSEL	0x03		/* 00=488Hz 01=976Hz 10=244Hz 11=122Hz */
#define IQW_TIMFFC	0x04		/* 1=clear ff */
#define IQW_IRQACK	0x08		/* 1=clear int */
#define IQW_TIMCLR	0x10
#define IQW_TIMEN	0x20
#define IQW_TIMIRQ	0x40		/* enable timer IRQ9 */

/* -------------------------------------------------------------------- */
#define DTC3STAT	0x160
#define MASK_rE160	0x01		/* T-error */
#define MASK_rCRES	0x02		/* C0RES, mirrors out 160 bit 0 */
#define MASK_rREPR	0x04		/* mirrors XREPR */
#define MASK_rIPIPE	0x10		/* from BDM */
#define DTC3ID		0x162

#define WR_CRES		0x160		/* bit 0 */
#define WR_ANAL		0x161

#define MEMSIZE		0x1000
#define MEMMASK		(MEMSIZE-1)

#define M68BOOT		0x440
#define M68STK		0x1f00
#define M68P_C		0x400
#define M68P_A		0x402
#define M68P_L		0x406
#define M68P_I		0x408
#define M68P_D		0x420

/* ==================================================================== */

#define MPU1		0x300
#define MPU2		0x302
#define MPU3		0x304
#define MPU4		0x306

/* mpu-401 command and message summary */

#define     NOTE_OFF            0x80    /* MIDI voice messages - channel 1 */
#define     NOTE_ON             0x90
#define     POLY_AFTERTOUCH     0xA0
#define     MODE_MESSAGE        0xB0
#define     PROGRAM_CHANGE      0xC0
#define     AFTERTOUCH          0xD0
#define     PITCH_WHEEL         0xE0
#define     ALL_NOTES_OFF       0x7B    /* channel mode message */

/* ==================================================================== */

#ifndef min
#define min(a,b) \
	({int _a=(a), _b=(b); _a < _b ? _a : _b; })
#endif
#ifndef max
#define max(a,b) \
	({int _a=(a), _b=(b); _a > _b ? _a : _b; })
#endif

static inline void IrqCtrlSet(const unsigned char s)
{
  __asm__ __volatile__ ("
	inb	%%dx,%%al
	orb	%1,%%al
	outb	%%al,%%dx"
	:
	: "d"(IRQCTRL), "g"(s)
	: "%eax" );
}

static inline void IrqCtrlSet_p(const unsigned char s)
{
  __asm__ __volatile__ ("
	pushfl
	cli
	inb	%%dx,%%al
	orb	%1,%%al
	outb	%%al,%%dx
	popfl"
	:
	: "d"(IRQCTRL), "g"(s)
	: "%eax" );
}

static inline void IrqCtrlClr(const unsigned char r)
{
  __asm__ __volatile__ ("
	inb	%%dx,%%al
	andb	%1,%%al
	outb	%%al,%%dx"
	:
	: "d"(IRQCTRL), "g"(~r)
	: "%eax" );
}

static inline void IrqCtrlClr_p(const unsigned char r)
{
  __asm__ __volatile__ ("
	pushfl
	cli
	inb	%%dx,%%al
	andb	%1,%%al
	outb	%%al,%%dx
	popfl"
	:
	: "d"(IRQCTRL), "g"(~r)
	: "%eax" );
}

static inline void IrqCtrlSetClr(const unsigned char s, const unsigned char r)
{
  __asm__ __volatile__ ("
	pushfl
	cli
	inb	%%dx,%%al
	andb	%1,%%al
	orb	%2,%%al
	outb	%%al,%%dx
	popfl"
	:
	: "d"(IRQCTRL), "g"(~r), "g"(s)
	: "%eax" );
}

extern char FPGAappl[];
extern char FPGAtype[];
extern char FPGAdate[];

int ResetFPGA(int boot);
int FPGAConfTab(void);
ssize_t fpga_write(struct file * file, const char * buf,
	size_t count, loff_t *ppos);

#endif	_LINUX_DTC3I_H

