
/************************************************************************
*									*
*	Device Driver for DTC3 AV 1998                                  *
*									*
*************************************************************************/

#ifndef MODULE
#error can only compile as a module
#endif

#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/major.h>
#include <linux/proc_fs.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/delay.h>

#include "dtc3_int.h"
#define HS_INTERFACE	/* for BDM */
#include "bdm.h"
#include "uart401.h"
#include "link.h"

#define CARD_NAME	"dtc3"
/*
 * devices:	0=base and M68k
 *		6=stat
 *		16,17,18,19=MIDI ports
 */

#include <linux/string.h>
#include <linux/vmalloc.h>

#undef  LINK_DEBUG 
#undef  LINK_DEBUG_MORE

#ifdef LINK_DEBUG
#define DEB(x)		x
#define PRINTK(f, a...)	printk(f, ##a)
#define NO_DEB(x)
#else
#define DEB(x)
#define PRINTK(f, a...)
#define NO_DEB(x)	x
#endif

#ifdef LINK_DEBUG_MORE
#define PRINTKM(f, a...)	printk(f, ##a)
#else
#define PRINTKM(f, a...)
#endif

unsigned long MemBase = 0xd0000;
int NoSetup = 0;
int ForceReset = 0;
int IrqNum = 9;
int UseMidi = 1;
int UseMidiIrq = 0;
int UseBDM = 1;

static int FPGArev = 0;
static int UseIrq = 9;
static int NumMidi, NumTransp, MemState;
static int MemErrAddr;
static char min_in_use[256];

static int use_schedule = 0;            /* don't use schedule() in init() */

static unsigned char TimFsel = 0;
unsigned long TimCount = 0;
unsigned long MidCount = 0;

static void ResetDtc3 (void);
static int SetupDtc3 (void);

struct wait_queue *dtc3wq = NULL;

extern void dtc3_irq (int irq, void *dev_id, struct pt_regs *regs);
extern int dtc3_midi_ioctl(int minor, struct file *file,
			    unsigned int cmd, unsigned long arg);


#define TCNT1	20000
#define TCNT2	10000

unsigned short M68Vects[] =
{
	0x0000,M68STK,0x0000,M68BOOT
};

unsigned short M68Pgm[] =
{
  0x387C,M68P_C,
  0x347C,M68P_L,
  0x3A7C,M68P_I,
  0x46FC,0x2700,
  0x33FC,0x0004,0x00FF,0xFA20,
  0x33FC,0x40CC,0x00FF,0xFA00,
  0x33FC,0xC008,0x00FF,0xFA04,
  0x33FC,0x0034,0x00FF,0xFA16,
  0x4279,0x00FF,0xFA14,
  0x4279,0x00FF,0xFA1E,
  0x33FC,0x0006,0x00FF,0xFA1C,
  0x33FC,0x3057,0x00FF,0xFA44,
  0x33FC,0x03C1,0x00FF,0xFA46,
  0x33FC,0x0005,0x00FF,0xFA48,
  0x33FC,0x7830,0x00FF,0xFA4A,
  0x33FC,0x0C01,0x00FF,0xFA60,
  0x33FC,0x7830,0x00FF,0xFA62,
  0x33FC,0x1007,0x00FF,0xFA70,
  0x33FC,0x7830,0x00FF,0xFA72,
  0x33FC,0x0004,0x00FF,0xFB04,
  0x33FC,0x1000,0x00FF,0xFB06,
  0x33FC,0x0200,0x00FF,0xFB00,
  0x33FC,0x0004,0x00FF,0xFB44,
  0x33FC,0x2000,0x00FF,0xFB46,
  0x33FC,0x0200,0x00FF,0xFB40,
  0x367C,M68P_A,
/*               	.L22:*/
  0x5295,      		/*addq.l #1,(%a5)*/
  0x3014,      		/*move.w (%a4),%d0*/
  0x0C40,0x0001,	/*cmp.w #1,%d0*/
  0x661C,      		/*jbne .L25*/
  0x2253,      		/*move.l (%a3),%a1*/
  0x307C,M68P_D, 	/*move.w #1056,%a0*/
  0x4A52,      		/*tst.w (%a2)*/
  0x6732,      		/*jbeq .L33*/
/*               	.L28:*/
  0x30D9,      		/*move.w (%a1)+,(%a0)+*/
  0x3012,      		/*move.w (%a2),%d0*/
  0x3200,      		/*move.w %d0,%d1*/
  0x5341,      		/*subq.w #1,%d1*/
  0x3481,      		/*move.w %d1,(%a2)*/
  0x0C40,0x0001, 	/*cmp.w #1,%d0*/
  0x66F0,      		/*jbne .L28*/
  0x6020,      		/*jbra .L33*/
/*               	.L25:*/
  0x0C40,0x0002, 	/*cmp.w #2,%d0*/
  0x66D4,      		/*jbne .L22*/
  0x2053,      		/*move.l (%a3),%a0*/
  0x327C,M68P_D,	/*move.w #1056,%a1*/
  0x4A52,      		/*tst.w (%a2)*/
  0x6710,      		/*jbeq .L33*/
/*               	.L34:*/
  0x30D9,      		/*move.w (%a1)+,(%a0)+*/
  0x3012,      		/*move.w (%a2),%d0*/
  0x3200,      		/*move.w %d0,%d1*/
  0x5341,      		/*subq.w #1,%d1*/
  0x3481,      		/*move.w %d1,(%a2)*/
  0x0C40,0x0001, 	/*cmp.w #1,%d0*/
  0x66F0,      		/*jbne .L34*/
/*               	.L33:*/
  0x4254,      		/*clr.w (%a4)*/
  0x60B6,      		/*jbra .L22*/
  0x4E71      		/*nop*/
};


/****************************************************************************/

MODULE_PARM(MemBase, "l");
MODULE_PARM(NoSetup, "i");
MODULE_PARM(ForceReset, "i");
MODULE_PARM(IrqNum, "i");
MODULE_PARM(UseMidi, "i");
MODULE_PARM(UseBDM, "i");

/****************************************************************************/


static void wr1link(char b, int port)
{
 int t=20000;
  while (((inb(port+3)&1)==0)&&(--t));
  if (t) outb(b,port+1);
}

static int rd1link(int port)
{
 int t=20000;
  while (((inb(port+2)&1)==0)&&(--t));
  return (t? inb(port):-1);
}

static void wr4link(unsigned long v, int port)
{
  wr1link(v&0xff,port); v>>=8;
  wr1link(v&0xff,port); v>>=8;
  wr1link(v&0xff,port); v>>=8;
  wr1link(v&0xff,port);
}

static unsigned long rd4link(int port)
{
 unsigned long v;
  v =(long)rd1link(port);
  v|=((long)rd1link(port)<<8);
  v|=((long)rd1link(port)<<16);
  v|=((long)rd1link(port)<<24);
  return(v);
}

#if 0
static int m68peek(unsigned short *buf,unsigned long a,int len)
{
  int i, l2;

  while (len) {
    l2 = (len>=16? 16:len);
    writew(l2, MemBase+M68P_L);
    writew((a>>16), MemBase+M68P_A);
    writew(a, MemBase+M68P_A+2);
    writew(1, MemBase+M68P_C);
    while (readw(MemBase+M68P_C)!=0);
    for (i=0; i<l2; i++) *buf++ = readw(MemBase+M68P_D+(i*2));
    len -= l2;
  }
  return 0;
}

static int m68poke(unsigned short *buf,unsigned long a,int len)
{
  int i, l2;

  while (len) {
    l2 = (len>=16? 16:len);
    for (i=0; i<l2; i++) writew(*buf++, MemBase+M68P_D+(i*2));
    writew(l2, MemBase+M68P_L);
    writew((a>>16), MemBase+M68P_A);
    writew(a, MemBase+M68P_A+2);
    writew(2, MemBase+M68P_C);
    while (readw(MemBase+M68P_C)!=0);
    len -= l2;
  }
  return 0;
}
#endif

/* -------------------------------------------------------------------- */

static unsigned char last_ctrlport = 0xff;


static unsigned char CtrlSet(unsigned char s, unsigned char r)
{
  long flags;
  register unsigned char b, c;

  save_flags(flags);
  cli();
  c = last_ctrlport;
  b = ((((inb(CTRLPORT)&~MASK_DRAM) | (c&MASK_DRAM)) & (~r)) | s);
  if (b != c) {
      outb(b, CTRLPORT); last_ctrlport = b;
#ifdef DEBUG
      printk("ctrl=%#x ",b);
#endif
  }
  restore_flags(flags);
  return c;
}

static inline void SelectMemBank (const int bank)
{
  if (bank) CtrlSet(MASK_A12, 0x00);
    else CtrlSet(0x00, MASK_A12);
}


/****************************************************************************/

#define D3PSLIMIT	(PAGE_SIZE-80)

int dtc3_read_procstat(char *buf, char **start, off_t offset,
	int len, int unused)
{
  static char ssep[] =
    "--------------------------------------------------------------------\n";
  int a;

	len=0;
	len += sprintf(buf+len,"%sDTC3 driver using major device %d\n",
			ssep, DTC3_MAJOR);
	len += sprintf(buf+len,"FPGA type %s created %s from file %s\n",
			FPGAtype, FPGAdate, FPGAappl);
	a = (inb(DTC3ID)<<8) | (inb(DTC3ID+1));
	len += sprintf(buf+len,"Id %04x rev %d - MemBase at %06x test codes %08x:%08x\n",
			(a & 0xfffc), (a & 3), MemBase, MemState, MemErrAddr);
	if (len > D3PSLIMIT) return len;

	if (NumTransp < 0) {
		len += sprintf(buf+len,"C012 err ");
	}
	else {
		len += sprintf(buf+len,"C012 ok, %d Transputers found ",
			NumTransp);
	}
	if (UseMidi) {
		len += sprintf(buf+len,"- %d MIDI found", NumMidi);
	}
	if (len > D3PSLIMIT) return len;

	len += sprintf(buf+len,"\nctl=%02x iqc=%02x iqs=%02x TimCount=%d MidCount=%d\n",
		inb(CTRLPORT), inb(IRQCTRL), inb(IRQSTAT), TimCount, MidCount);
	if (len > D3PSLIMIT) return len;

	len += sprintf(buf+len,"%s", ssep);
	return len;
}


struct proc_dir_entry dtc3_proc_entry =
{
	0,			/* low_ino: the inode--dynamic */
	4, "dtc3",		/* len of name and name */
	S_IFREG | S_IRUGO,	/* mode */
	1, 0, 0,		/* nlinks, owner, group */
	0,			/* size--unused */
	NULL,			/* operations--use default */
	&dtc3_read_procstat,	/* function */
};


/****************************************************************************/

static int link_devices = 0;

static int link_base_addresses[LINK_NO] = { 0x550, 0x150 };

static int link_boot_stat[LINK_NO] = { LINK_STATUS_RESET, LINK_STATUS_RESET };

/*
 * Struct to describe the default parameters of each link interface
 */
static struct link_struct link_table[] =
	{
		 { LINK_READ_ABORT | LINK_WRITE_ABORT, 0, 0, 0, 0,
		 LINK_INIT_READ_TIMEOUT, LINK_INIT_WRITE_TIMEOUT, LINK_B004
		 },
		 { LINK_READ_ABORT | LINK_WRITE_ABORT, 0, 0, 0, 0,
		 LINK_INIT_READ_TIMEOUT, LINK_INIT_WRITE_TIMEOUT, LINK_B004
		 },
		 { LINK_READ_ABORT | LINK_WRITE_ABORT, 0, 0, 0, 0,
		 LINK_INIT_READ_TIMEOUT, LINK_INIT_WRITE_TIMEOUT, LINK_B004
		 },
	};

static inline void link_call_sched_if (void)
{
	if (current->need_resched) schedule();
}


/****************************************************************************
 *
 * static void link_reset():
 *
 * Reset transputer network.
 *
 */
static void link_reset(const int minor)
{
	PRINTK("LINK(%d) resetting transputer.\n", minor);

	outb(LINK_DEASSERT_ANALYSE, LINK_ANALYSE(minor));
	udelay(LINK_DELAY);

	outb(LINK_DEASSERT_RESET, LINK_RESET(minor));
	udelay(LINK_DELAY);

	outb(LINK_ASSERT_RESET, LINK_RESET(minor));
	udelay(LINK_DELAY);

	outb(LINK_DEASSERT_RESET, LINK_RESET(minor));
	udelay(LINK_DELAY);

	link_boot_stat[minor] = LINK_STATUS_RESET;

} /* link_reset() */

/****************************************************************************
 *
 * static void link_analyse():
 *
 * switch transputer network to analyse mode.
 *
 */
static void link_analyse( const int minor )
{
	PRINTK("LINK(%d) switching transputer to analyse mode.\n", minor);

	outb( LINK_DEASSERT_ANALYSE, LINK_ANALYSE(minor));
	udelay(LINK_DELAY);

	outb( LINK_ASSERT_ANALYSE, LINK_ANALYSE(minor));
	udelay(LINK_DELAY);

	outb( LINK_ASSERT_RESET, LINK_RESET(minor));
	udelay(LINK_DELAY);

	outb( LINK_DEASSERT_RESET, LINK_RESET(minor));
	udelay(LINK_DELAY);

	outb( LINK_DEASSERT_ANALYSE, LINK_ANALYSE(minor));
	udelay(LINK_DELAY);

	link_boot_stat[minor] = LINK_STATUS_RESET | LINK_STATUS_ANALYSE;

} /* link_analyse() */


/****************************************************************************
 *
 */

static int link_peek (int minor, u_long addr, u_long *data)
{
	unsigned char s[8];
	int i;

	if (link_boot_stat[minor] == 0) return 1;

	s[0]=1;
	*((u_long *)(s+1)) = addr;

	for (i=0; i<5; i++) {
		while (!inb(LINK_OSR(minor)) & LINK_WRITEBYTE)
			link_call_sched_if();
		outb(s[i], LINK_ODR(minor));
	}
	for (i=0; i<4; i++) {
		while (!inb(LINK_ISR(minor)) & LINK_READBYTE)
			link_call_sched_if();
		((char *)data)[i] = inb(LINK_IDR(minor));
	}
	return 0;
}
 

static int link_poke (int minor, u_long addr, u_long data)
{
	unsigned char s[12];
	int i;

	if (link_boot_stat[minor] == 0) return 1;

	s[0]=0;
	*((u_long *)(s+1)) = addr;
	*((u_long *)(s+1)) = data;

	for (i=0; i<9; i++) {
		while (!inb(LINK_OSR(minor)) & LINK_WRITEBYTE)
			link_call_sched_if();
		outb(s[i], LINK_ODR(minor));
	}
	return 0;
}
 

/****************************************************************************
 *
 * static int link_read() - read bytes from the link interface.
 *
 * At first, the driver checks if the link-interface is ready to send a byte
 * to the PC. If not, this check is repeated up to LINK_MAXTRY times.
 * If the link-interface is not ready after this loop, the driver sleeps for
 * LINK_TIME jiffies and then checks the link-interface again.
 * If the interface is still not ready, the driver sleeps for LINK_READ_TIMEOUT
 * jiffies. Then it checks the interface again. If it is not ready and
 * the flag LINK_READ_ABORT is set, this function returns with the number
 * of bytes read or with an error if no byte was read.
 * If the LINK_READ_ABORT flag is not set, the driver sleeps again for
 * LINK_READ_ABORT jiffies and so on.
 *
 * By default, LINK_READ_ABORT is now set.
 *
 */
static ssize_t link_read(struct file *filp, char *buf, size_t count,
		loff_t *ppos)
{
	const unsigned int minor = MINOR(filp->f_dentry->d_inode->i_rdev);
	unsigned int read_sleep = LINK_START_SLEEP;
	unsigned int read_wait  = 0;
	unsigned int read_inc = 0;
	DEB( unsigned int link_total_bytes_read = 0; )
	char 	*temp = buf;
	int	l_count = 0;
	char	buffer[LINK_MAX_BYTES];
	int	i;

	PRINTKM("LINK(%d): reading %d bytes (polling mode).\n", minor, count);

	if (link_boot_stat[minor] == LINK_STATUS_RESET) {
	  u_long k;
	  if (link_peek (minor, *ppos, &k)) return -EINVAL;
	  copy_from_user(&k, buf, 4);
	  *ppos += 4;
	  return 4;
	}

	if ( count < 0)
	{
		PRINTK("LINK: invalid argument for reading: count = %d.\n", count);
		return -EINVAL;
	}

	while ( count )
	{
		int end = (LINK_MAX_BYTES > count) ? count : LINK_MAX_BYTES;
		l_count = 0;
		do {
		    if (!inb(LINK_ISR(minor)) & LINK_READBYTE)
		    {
			i = LINK_MAXTRY;
			while (!(inb(LINK_ISR(minor)) & LINK_READBYTE) && i) i--;
			if (!i)	break;
		    }
		    buffer[l_count++] = (char) inb(LINK_IDR(minor));
		    DEB(link_total_bytes_read++;)
		} while (--end);
		if (l_count)
		{
			copy_to_user(temp, buffer, l_count);
			count -= l_count;
			temp += l_count;
			link_call_sched_if();
		}
		else while ( !( inb(LINK_ISR(minor)) & LINK_READBYTE ) )
		{
			/* check for signals before going to sleep. */
			if (signal_pending(current)) {
				if ((temp - buf)==0) return -EINTR;
				goto rquit;
			}

			PRINTK("LINK(%d) sleeping at %d bytes for %d jiffies (reading).\n",
					minor, link_total_bytes_read, read_sleep );

			current->state = TASK_INTERRUPTIBLE;
			schedule_timeout(read_sleep);

			read_wait += read_sleep;
			read_inc = (read_inc + 1) % LINK_INC;
			read_sleep += (read_sleep < LINK_MAX_SLEEP && !read_inc);

			if ( read_wait > LINK_READ_TIMEOUT(minor)
				&& (LINK_F(minor) & LINK_READ_ABORT) )
			{
				PRINTK("LINK(%d): timeout during read.\n", minor);
				goto rquit;
			}
				
		}
	}
	PRINTKM("LINK(%d) reading successful.\n", minor);
rquit:
	*ppos += (temp - buf);
	return temp - buf;

} /* LINK_read() */



/****************************************************************************
 *
 * static int link_write() - write to the link interface.
 *
 */
static ssize_t link_write(struct file *filp, const char *buf, size_t count,
		loff_t *ppos)
{
	const unsigned int minor = MINOR(filp->f_dentry->d_inode->i_rdev);
	unsigned int write_sleep = LINK_START_SLEEP;
	unsigned int write_wait  = 0;
	unsigned int write_inc = 0;
	DEB( unsigned int link_total_bytes_written = 0; )
	char	*temp = (char *)buf;
	int	l_count = 0;
	char	buffer[LINK_MAX_BYTES];
	int	i;

	PRINTKM("LINK(%d): writing %d bytes (polling mode).\n", minor, count);

	if (count < 0)
	{
		PRINTK("LINK invalid argument for writing: count = %d.\n", count);
		return -EINVAL;
	}

	while (count)
	{
		int end = (LINK_MAX_BYTES > count) ? count : LINK_MAX_BYTES;
		copy_from_user(buffer, temp, end);
		l_count = 0;
		do {
		    if (!inb(LINK_OSR(minor)) & LINK_WRITEBYTE)
		    {
			i = LINK_MAXTRY;
			while (!(inb(LINK_OSR(minor)) & LINK_WRITEBYTE) && i) i--;
			if (!i)	break;
		    }
		    outb(buffer[l_count++], LINK_ODR(minor));
		    DEB(link_total_bytes_written++;)
		} while (--end);
		if (l_count)
		{
			temp += l_count;
			count -= l_count;
			link_call_sched_if();
		}
		else while ( !( inb(LINK_OSR(minor)) & LINK_WRITEBYTE ) )
		{
			/* check for signals before going to sleep. */
			if (signal_pending(current))
			{
				if (temp-buf) goto wquit; else return -EINTR;
			}

			PRINTK("LINK(%d) sleeping at %d bytes for %d jiffies (writing).\n",
					minor, link_total_bytes_written, write_sleep );

			current->state = TASK_INTERRUPTIBLE;
			schedule_timeout(write_sleep);

			write_wait += write_sleep;
			write_inc = (write_inc + 1) % LINK_INC;
			write_sleep += (write_sleep < LINK_MAX_SLEEP && !write_inc);

			if ( write_wait > LINK_WRITE_TIMEOUT(minor)
				&& (LINK_F(minor) & LINK_WRITE_ABORT) )
			{
				PRINTK("LINK(%d): timeout during write.\n", minor);
				goto wquit;
			}
				
		}
	}

	PRINTKM("LINK(%d): write successful.\n", minor);

wquit:
	link_boot_stat[minor] = LINK_STATUS_BOOTED;
	*ppos += (temp - buf);
	return temp-buf;
} /* link_poll_write() */

/****************************************************************************
 *
 * static int link_lseek()
 *
 */
static loff_t link_lseek(struct file * file, loff_t offset, int origin)
{
	switch (origin) {
		case 0:
			file->f_pos = offset;
			return file->f_pos;
		case 1:
			file->f_pos += offset;
			return file->f_pos;
		default:
			return -EINVAL;
	}
	if (file->f_pos < 0) return 0;
	return file->f_pos;
}


/****************************************************************************
 *
 * static int link_release()
 *
 * close the link device.
 *
 */
static int link_release(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);

	if (minor > 15) return 0;

	if (minor >= link_devices)
	{
		PRINTK("LINK(%d) not released, minor device number >= %d.\n",
			minor, link_devices);
		return -ENODEV;
	}

	LINK_F(minor) &= ~LINK_BUSY;

	MOD_DEC_USE_COUNT;

	PRINTK("LINK(%d) released.\n", minor);
	return 0;

} /* link_release() */


/****************************************************************************
 *
 * static int link_ioctl()
 *
 * This function performs the various ioctl() functions: resetting the
 * transputer, switching to analyse-mode, testing the status, changing
 * timeouts etc.
 *
 */
static int link_ioctl(struct inode *inode, struct file *file,
		    unsigned int cmd, unsigned long arg)
{
	const unsigned int minor = MINOR(inode->i_rdev);
	int result = arg;
	u_long k;

	PRINTK("LINK(%d) ioctl, cmd: 0x%08x, arg: 0x%08x.\n", minor, cmd, (int) arg);

	if (minor > 15) return -EINVAL;

	if ((minor >= link_devices) || !(LINK_F(minor) & LINK_BUSY) )
	{
		DEB(
			if (minor >= link_devices)
				printk("LINK ioctl exit, minor >= %d.\n", link_devices );
			else
				printk("LINK ioctl exit, device not opened.\n" );
		)
		return -ENODEV;
	}

	switch (cmd)
	{
		case LINKRESET:		/* reset transputer */
			link_reset(minor);
			break;
		case LINKWRITEABLE:	/* can we write a byte to the C012 ? */
	 		return ( ( inb(LINK_OSR(minor)) & LINK_WRITEBYTE) != 0 ); 
		case LINKREADABLE:	/* can we read a byte from C012 ? */
	 		return ( ( inb(LINK_ISR(minor)) & LINK_READBYTE) != 0 ); 
		case LINKANALYSE:	/* switch transputer to analyse mode */
			link_analyse(minor);
			break;
		case LINKERROR:		/* test error-flag */
			return ( inb(LINK_BASE(minor) + LINK_ERROR_OFFSET) & LINK_TEST_ERROR) ? 0 : 1;
		case LINKREADTIMEOUT:	/* set timeout for reading */
			result = LINK_READ_TIMEOUT(minor);
			LINK_READ_TIMEOUT(minor) = arg;
			break;
		case LINKWRITETIMEOUT:	/* set timeout for writing */
			result = LINK_WRITE_TIMEOUT(minor);
			LINK_WRITE_TIMEOUT(minor) = arg;
			break;
		case LINKREADABORT:	/* abort after a timeout ? */
			if ( arg )
				LINK_F(minor) |= LINK_READ_ABORT;
			else
				LINK_F(minor) &= ~LINK_READ_ABORT;
			break;
		case LINKWRITEABORT:	/* abort after a timeout ? */
			if ( arg )
				LINK_F(minor) |= LINK_WRITE_ABORT;
			else
				LINK_F(minor) &= ~LINK_WRITE_ABORT;
			break;
		case LINKFASTREAD:
			if (inb(LINK_ISR(minor)) & LINK_READBYTE)
			  result = (unsigned char) inb(LINK_IDR(minor));
			else
			  result = -EBUSY;
			break;
		case LINKFASTWRITE:
			if (inb(LINK_OSR(minor)) & LINK_WRITEBYTE) {
			  outb((unsigned char) arg, LINK_ODR(minor));
			  link_boot_stat[minor] = LINK_STATUS_BOOTED;
			  result = 0;
			}
			else
			  result = -EBUSY;
			break;
		case LINKPEEK:
			if (link_peek (minor, file->f_pos, &k))
				return -EINVAL;
			file->f_pos += 4;
			result = k;
			break;
		case LINKPOKE:
			if ((result=link_poke (minor, file->f_pos, arg)))
				return -EINVAL;
			file->f_pos += 4;
			break;
		case LINKGETSTAT:
			result = link_boot_stat[minor];
			break;
		default: result = -EINVAL;
	}

	PRINTK("LINK(%d) ioctl done.\n", minor);

	return result;

} /* link_ioctl() */


/****************************************************************************/
/* 2.1.60:
 * struct file_operations {
 *	loff_t (*llseek) (struct file *, loff_t, int);
 *	ssize_t (*read) (struct file *, char *, size_t, loff_t *);
 *	ssize_t (*write) (struct file *, const char *, size_t, loff_t *);
 *	int (*readdir) (struct file *, void *, filldir_t);
 *	unsigned int (*poll) (struct file *, poll_table *);
 *	int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
 *	int (*mmap) (struct file *, struct vm_area_struct *);
 *	int (*open) (struct inode *, struct file *);
 * 2.1.118:
 *	int (*flush) (struct file *);
 *	int (*release) (struct inode *, struct file *);
 *	int (*fsync) (struct file *, struct dentry *);
 *	int (*fasync) (struct file *, int);
 *	int (*check_media_change) (kdev_t dev);
 *	int (*revalidate) (kdev_t dev);
 *	int (*lock) (struct file *, int, struct file_lock *);
 * };
 */

static struct file_operations link_fops = {
	link_lseek,
	link_read,
	link_write,
	NULL,		/* readdir */
	NULL,		/* poll */
	link_ioctl,
	NULL,		/* mmap */
	NULL,		/* open */
	NULL,		/* flush */
	link_release,
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};


/****************************************************************************
 *
 * static int link_open()
 *
 * open the link-device.
 *
 */
static int link_open(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);

	if (minor >= link_devices)
	{
		PRINTK("LINK(%d) not opened, minor device number >= %d.\n",
			minor, link_devices);
		return -ENODEV;
	}

	if (LINK_F(minor) & LINK_BUSY)
	{
		PRINTK("LINK(%d) not opened, device busy.\n", minor);
		return -EBUSY;
	}

	LINK_F(minor) |= LINK_BUSY;
	filp->f_op = &link_fops;

	MOD_INC_USE_COUNT;

	PRINTK( "LINK(%d) opened.\n", minor);
	return 0;

} /* link_open() */


/****************************************************************************
 *
 * link_fops
 *
 * File operations
 */
static struct file_operations dtc2l_fops = {
	NULL,		/* llseek */
	NULL,		/* read */
	NULL,		/* write */
	NULL,		/* readdir */
	NULL,		/* poll */
	NULL,		/* ioctl */
	NULL,		/* mmap */
	link_open,
	NULL,		/* flush */
	NULL,		/* release */
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};


/****************************************************************************
 *
 * long link_init()
 *
 * This function initializes the driver. It tries to detect the hardware
 * and sets up all relevant data structures.
 *
 */
static int link_init(void)
{
	unsigned int test = 0, i;
	unsigned char link_dev = 0;

	/*
	   After a reset it should be possible to write a byte to
	   the LINK. So let's do a reset and then test the output status
	   register
	*/
	for (; link_base_addresses[test] && link_dev < LINK_NO; test++)
	{
		udelay(LINK_DELAY * 10);
		LINK_BASE(link_dev) = link_base_addresses[test];
		LINK_ODR(link_dev) = LINK_BASE(link_dev) + LINK_ODR_OFFSET;
		LINK_ISR(link_dev) = LINK_BASE(link_dev) + LINK_ISR_OFFSET;
		LINK_OSR(link_dev) = LINK_BASE(link_dev) + LINK_OSR_OFFSET;

		link_reset(link_dev);

		udelay(LINK_DELAY * 10);

		for (i = 0; i < LINK_MAXTRY; i++) if ( inb(LINK_OSR(link_dev)) == LINK_WRITEBYTE)
		{
#if 0
			outb(B008_INT(link_dev), 0);
			udelay(LINK_DELAY * 10);
			LINK_BOARDTYPE(link_dev) =
				(inb(B008_INT(link_dev)) & 0x0f) == 0 ?
					LINK_B008 : LINK_B004;
/*restart_init:*/
			/* LINK_BOARDTYPE(link_dev) = LINK_B008; */
#else
			LINK_BOARDTYPE(link_dev) = LINK_B004;
#endif
			printk("link%d at 0x0%x (polling) is a B00%c\n",
				link_dev,LINK_IDR(link_dev),
				LINK_BOARDTYPE(link_dev));
			link_dev++;
			break;
		}

	}

	link_devices = link_dev;

	if (!link_dev) {
		printk("link: no interfaces found.\n");
		return 1;
	}

	return 0;
}



/****************************************************************************/


static int dtc3_release(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);
	int e = 0;

	if (min_in_use[minor] <= 0) return -ENAVAIL;

	if (UseMidi && (minor >= MINOR_MIDI) && (minor < MINOR_MAXMIDI)) {
		if (minor >= (MINOR_MIDI+NumMidi)) return -ENODEV;
		e = dtc3_midi_release(minor-MINOR_MIDI);
		if (e) return -EIO;
	}
	else switch (minor) {
	  case MINOR_DTC3:
		SelectMemBank(0);
		break;
	  case MINOR_FPGA:
	  	break;
	  case MINOR_BDM:
	  	break;
	  case MINOR_TIM:
	  	break;
	  default: return -ENODEV;
	}

	min_in_use[minor] -= 1;
	MOD_DEC_USE_COUNT;
	return e;
}


/*****************************************************************************/
/* 2.1.60:
 * struct file_operations {
 *	loff_t (*llseek) (struct file *, loff_t, int);
 *	ssize_t (*read) (struct file *, char *, size_t, loff_t *);
 *	ssize_t (*write) (struct file *, const char *, size_t, loff_t *);
 *	int (*readdir) (struct file *, void *, filldir_t);
 *	unsigned int (*poll) (struct file *, poll_table *);
 *	int (*ioctl) (struct inode *, struct file *, unsigned int, unsigned long);
 *	int (*mmap) (struct file *, struct vm_area_struct *);
 *	int (*open) (struct inode *, struct file *);
 * 2.1.118:
 *	int (*flush) (struct file *);
 *	int (*release) (struct inode *, struct file *);
 *	int (*fsync) (struct file *, struct dentry *);
 *	int (*fasync) (struct file *, int);
 *	int (*check_media_change) (kdev_t dev);
 *	int (*revalidate) (kdev_t dev);
 *	int (*lock) (struct file *, int, struct file_lock *);
 * };
 */

static int dtc3_open(struct inode *inode, struct file *filp)
{
	const unsigned int minor = MINOR(inode->i_rdev);
	int e = 0;

	if (min_in_use[minor]) return -ENAVAIL;

	if (UseMidi && (minor >= MINOR_MIDI) && (minor < MINOR_MAXMIDI)) {
		if (minor >= (MINOR_MIDI+NumMidi)) return -ENODEV;
		e = dtc3_midi_open(minor-MINOR_MIDI);
		if (e) return -EIO;
	}
	else switch (minor) {
	  case MINOR_DTC3:
	  	break;
	  case MINOR_FPGA:
	  	break;
	  case MINOR_BDM:
	  	break;
	  case MINOR_TIM:
	  	break;
	  default: return -ENODEV;
	}

	min_in_use[minor] += 1;
	MOD_INC_USE_COUNT;
	return e;
}

/****************************************************************************/


static loff_t dtc3_lseek(struct file * file, loff_t offset, int orig)
{
	unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);

	if (minor > MINOR_BDM)
		return -ENODEV;
	switch (orig) {
		case 0:
			file->f_pos = offset;
			return file->f_pos;
		case 1:
			file->f_pos += offset;
			return file->f_pos;
		default:
			return -EINVAL;
	}
}


static ssize_t dtc3_write(struct file * file, const char * buf,
		        size_t count, loff_t *ppos)
{
	unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);

	if (UseMidi && (minor>=MINOR_MIDI) && (minor<(NumMidi+MINOR_MIDI))) {
		return dtc3_midi_write(minor-MINOR_MIDI,file,buf,count,ppos);
	}
	if (minor==MINOR_DTC3) {
		unsigned long p = (*ppos) & ~1;
		unsigned long end_mem = 2*MEMSIZE;
		unsigned short *q = (unsigned short *)buf;
		ssize_t written;
		int i;

		if (p > end_mem)
	  		return 0;
		if (count & 1) count--;
		if (count > (end_mem - p))
			count = end_mem - p;
		written = 0;
		SelectMemBank((p >= MEMSIZE));
		for (i=0; i<count; i+=2) {
			register unsigned short w;
			__get_user(w, q++);
			writew(w, (MemBase+(p&MEMMASK)));
			p += 2;
			if (p == MEMSIZE) SelectMemBank(1);
		}
		written += i;
		*ppos += i;
		return written;
	}
	else if (minor==MINOR_FPGA) {
		return fpga_write(file,buf,count,ppos);
	}
	else if (minor==MINOR_TIM)
	  	return -ENAVAIL;

	return -ENXIO;
}


static ssize_t dtc3_read(struct file * file, char * buf,
		       size_t count, loff_t *ppos)
{
	unsigned int minor=MINOR(file->f_dentry->d_inode->i_rdev);

	if (UseMidi && (minor>=MINOR_MIDI) && (minor<(NumMidi+MINOR_MIDI))) {
		return 0;
	}
	if (minor==MINOR_DTC3) {
		unsigned long p = (*ppos) & ~1;
		unsigned long end_mem = 2*MEMSIZE;
		unsigned short *q = (unsigned short *)buf;
		ssize_t read;
		int i;

		if (p > end_mem)
	  		return 0;
		if (count & 1) count--;
		if (count > (end_mem - p))
			count = end_mem - p;
		read = 0;
		SelectMemBank((p >= MEMSIZE));
		for (i=0; i<count; i+=2) {
			__put_user(readw(MemBase+(p&MEMMASK)), q++);
			p += 2;
			if (p == MEMSIZE) SelectMemBank(1);
		}
		read += i;
		*ppos += i;
		return read;
	}
	else if (minor==MINOR_FPGA) {
		if (count) {
			__put_user(inb(FPGA), buf);
			count = 1;
		}
		return count;
	}
	else if (minor==MINOR_TIM) {
		interruptible_sleep_on(&dtc3wq);
		return 0;
	}

	return -ENXIO;
}

	
/****************************************************************************/


static int dtc3_ioctl(struct inode *inode, struct file *file,
		    unsigned int cmd, unsigned long arg)
{
	unsigned int minor = MINOR(inode->i_rdev);
	int retval = 0;

	if (UseMidi && (minor>=MINOR_MIDI) && (minor<(NumMidi+MINOR_MIDI))) {
		return dtc3_midi_ioctl(minor-MINOR_MIDI,file,cmd,arg);
	}
	if (minor == MINOR_DTC3)
	  switch ( cmd ) {
		case DTC3_IDENT:
			retval =  (inb(DTC3ID)<<8)|inb(DTC3ID+1);
			break;
		case DTC3_RESET:
			ResetDtc3();	/* !! sets MIDI off */
			break;
		case DTC3_SETUP:
			retval =  SetupDtc3();
			break;

		case DTC3_MIDIIRQ:
			if (!UseMidi || (UseIrq <= 0)) return -EINVAL;
			UseMidiIrq = (arg & 1);
			if (UseMidiIrq) MidCount = 0;
			break;

		case DTC3_SXIN:
			retval =  inb(SXINPORT) & 0x0f;
			break;
		case DTC3_EXOUT:
			if (UseMidi) return -EINVAL;
			CtrlSet(((~arg)&0x0f),0x0f);
			break;
		default:
			retval = -EINVAL;
	  }
	else if (minor == MINOR_FPGA)
	  switch ( cmd) {
		case DTC3_FPGA_CLR:
			if (arg!=0x5a) return -EINVAL;
			if (UseIrq > 0) {
				CtrlSet(0x00,MASK_IRQON);
				free_irq(UseIrq, NULL);
			}
			retval = ResetFPGA(0);
			break;
		case DTC3_FPGA_PGM:
			/* FPGA must be cleared (DONE=0,INIT=1) */
			if ((inb(FPGA)&(DONE|INIT)) != INIT) return -EIO;

			retval = FPGAConfTab() & (DONE|nERROR);

			if (retval == (DONE|nERROR)) {
				/* test if FPGA programmed */
				if ((inb(DTC3ID)!=0xd3) || ((inb(DTC3ID+1)&0xfc)!=0x2c)) return -EIO;
				ResetDtc3();
			}
			break;
		case DTC3_FPGA_APPL:
			strncpy_from_user(FPGAappl, (void *)arg, 16);
			break;
		case DTC3_FPGA_TYPE:
			strncpy_from_user(FPGAtype, (void *)arg, 16);
			break;
		case DTC3_FPGA_DATE:
			strncpy_from_user(FPGAdate, (void *)arg, 30);
			break;
		case DTC3_FPGA_TEST:
			retval =  SetupDtc3();
			break;
		default:
			retval = -EINVAL;
	  }
	else if (minor == MINOR_TIM)
	  switch ( cmd ) {
		case DTC3_TIMFSEL:
			TimFsel = arg & 3;
			IrqCtrlClr_p(IQW_TIMEN);
			outb((inb(IRQCTRL)&0xfc)|TimFsel, IRQCTRL);
			break;
		case DTC3_TIMEN:
			if (arg & 1) {
				IrqCtrlSet_p(IQW_TIMEN|IQW_TIMCLR);
				IrqCtrlClr_p(IQW_TIMCLR);
			}
			else
				IrqCtrlClr_p(IQW_TIMEN);
			break;
		case DTC3_TIMCLR:
			if (inb(IRQCTRL) & IQW_TIMEN) {
				IrqCtrlSet_p(IQW_TIMCLR);
				IrqCtrlClr_p(IQW_TIMCLR);
			}
			break;
		case DTC3_TIMIRQ:
			if (UseIrq <= 0) return -EINVAL;
			if (arg & 1) {
				IrqCtrlSet_p(IQW_TIMIRQ);
				TimCount = 0;
			}
			else {
				IrqCtrlClr_p(IQW_TIMIRQ);
			}
			break;
		default:
			retval = -EINVAL;
	  }
	else
	  retval = -ENODEV;

	return retval;
}


/****************************************************************************/

static struct file_operations dtc3_fops = {
	dtc3_lseek,	/* llseek */
	dtc3_read,	/* read */
	dtc3_write,	/* write */
	NULL,		/* readdir */
	NULL,		/* poll */
	dtc3_ioctl,	/* ioctl */
	NULL,		/* mmap */
	dtc3_open,
	NULL,		/* flush */
	dtc3_release,
	NULL,		/* fsync */
	NULL,		/* fasync */
	NULL,		/* check_media_change */
	NULL,		/* revalidate */
	NULL		/* lock */
};


/****************************************************************************
 *
 * This function initializes the driver. It tries to detect the hardware
 * and sets up all relevant data-structures.
 *
 ****************************************************************************/


int DTCfound(void)
{
  unsigned char b,c;

  c = inb(FPGA);
  b = inb(0x80);
  outb (1, FPGA);
  if (c != inb(FPGA)) return 0;		/* floating */
  c = inb(FPGA);

  /* DONE low = FPGA not programmed (startup) */
  if ((c&(DONE|INIT))==INIT) return 1;
  /* DONE high with error, must be reset */
  if ((c&(DONE|INIT))==DONE) return 1;

  /* DONE and INIT both high, test if FPGA programmed */
  if ((inb(DTC3ID)!=0xd3) || ((inb(DTC3ID+1)&0xfc)!=0x2c)) return 0;
  return 2;
}


static unsigned int R_test1(unsigned short ptn)
{
  int i;
  register volatile unsigned short *pw = (unsigned short *)phys_to_virt(MemBase);
  register unsigned short v;

  for (i=0; i<MEMSIZE/2; i++) pw[i] = ptn;
  for (i=0; i<MEMSIZE/2; i++)
	if ((v=pw[i]) != ptn) { MemErrAddr=i; return 0xf0000000|v; }
  return 0;
}

static unsigned int R_test2(void)
{
  int i;
  register volatile unsigned short *pw = (unsigned short *)phys_to_virt(MemBase);
  register unsigned short v;

  for (i=0; i<MEMSIZE/2; i++) pw[i] = ~i;
  for (i=0; i<MEMSIZE/2; i++)
	if ((v=pw[i]) != (~i&0xffff)) { MemErrAddr=i; return 0xf0000000|v; }
  return 0;
}


static int M_test(int port)
{
  outb(0x3f,port+1);
  if ((inb(port+1)&0xc0)!=0) return 0;
  if (inb(port)!=0xfe) return 0;
  if ((inb(port+1)&0xc0)!=0x80) return 0;
  return 1;
}


static int T_test(int port)
{
 unsigned long u,v,a;
 int i;

  u=0x55aa5aa5;
  wr1link(0,port); wr4link(0x80000080,port); wr4link(u,port);	/* poke */
  wr1link(1,port); wr4link(0x80000080,port); v=rd4link(port);
  if (u!=v) return 1;
  u=0xaa334679;
  wr1link(0,port); wr4link(0x80000800,port); wr4link(u,port);	/* poke */
  wr1link(1,port); wr4link(0x80000800,port); v=rd4link(port);
  if (u!=v) return 1;
  a = 0x80001000;
  for (i=0; i<256; i++) {
      u = ~(u+1);
      wr1link(0,port); wr4link(a,port); wr4link(u,port); /* poke */
      wr1link(1,port); wr4link(a,port); v=rd4link(port);
      if (u!=v) return 1;
      a += 4;
  }
  return 0;
}

static void ResetC012(void)
{
  outb(1, WR_CRES);
}

static int ResetTransp(void)
{
  int nT;

  /* normal link order */
  CtrlSet(0x00, MASK_XC12);

  outb(0, WR_ANAL);
  outb(1, WR_CRES);	/* reset C012 */
  if ((inb(DTC3STAT)&MASK_rCRES)==0) return -1;
  udelay(1000);
  outb(0, WR_CRES);	/* unreset C012 */
  if ((inb(DTC3STAT)&MASK_rCRES)==MASK_rCRES) return -1;
  udelay(1000);
  if ((inb(C012A+2)!=0)&&(inb(C012A+3)!=1)) return 0;	/* LkA.150 is 0,1 */
  if ((inb(C012B+2)!=0)&&(inb(C012B+3)!=1)) return 0;	/* LkB.550 is 0,1 */
  outb(2,C012A+2);
  outb(2,C012A+3);					/* LkA.150 is 2,3 */
  if ((inb(C012A+2)!=2)&&(inb(C012A+3)!=3)) return 0;

  /* reverse link order */
  CtrlSet(MASK_XC12, 0x00);
  if ((inb(C012B+2)!=2)&&(inb(C012B+3)!=3)) return 0;	/* LkB.150 is 2,3 */
  outb(0,C012B+2);
  outb(0,C012B+3);					/* LkB.150 is 0,1 */
  outb(2,C012A+2);
  outb(2,C012A+3);					/* LkA.550 is 2,3 */

  /* normal link order */
  CtrlSet(0x00, MASK_XC12);
  if ((inb(C012B+2)!=2)&&(inb(C012B+3)!=3)) return 0;	/* LkB.550 is 2,3 */
  outb(0,C012B+2);
  outb(0,C012B+3);					/* LkB.150 is 0,1 */

  nT=1;
  if (!T_test(C012A)) nT++;
  if (!T_test(C012B)) nT++;

  ResetC012();
  return nT;
}


void ResetBDM(void)
{
  outb(BDM_RST_OUT, BDM_CTRL(0));
}


int ResetMem(void)
{
  register volatile unsigned short *pw = (unsigned short *)phys_to_virt(MemBase);
  int i;

  SelectMemBank(0);
  for (i=0; i<(sizeof(M68Vects)/2); i++)
	pw[i] = M68Vects[i];
  pw += (M68BOOT/2);
  for (i=0; i<(sizeof(M68Pgm)/2); i++)
	pw[i] = M68Pgm[i];
  return 0;
}


int SetupMem(void)
{
  unsigned int e, k;
  register volatile unsigned short *pw;
  register unsigned short v;
  unsigned long mb0, flags;

  mb0 = (0xc0+(inb(CTRLPORT)&MASK_DRAM))<<12;
  MemErrAddr = 0;
  printk(KERN_INFO "Base memory is at %#lx\n",MemBase);
  if (mb0 != MemBase)
	printk(KERN_ERR "WARNING - FPGA says %#lx\n", mb0);
  pw = (unsigned short *)phys_to_virt(MemBase);

  e = 0;
  save_flags(flags);
  cli();
  SelectMemBank(0);
  *pw = 0x1234;
  if ((v=*pw)!=0x1234) {
	restore_flags(flags);
  	e=0x100000|v; goto quit;
  }
  SelectMemBank(1);
  if ((v=*pw)==0x1234) {
	restore_flags(flags);
  	e=0x180000|v; goto quit;
  }
  restore_flags(flags);

  if ((v=R_test1(0x5a5a))) { e=0x200000|v; goto quit; }
  if ((v=R_test1(0xa5a5))) { e=0x300000|v; goto quit; }
  if ((v=R_test1(0xffff))) { e=0x400000|v; goto quit; }
  if ((v=R_test2())) { e=0x500000|v; goto quit; }
  if ((v=R_test1(0x0000))) { e=0x600000|v; goto quit; }
  save_flags(flags);
  cli();
  SelectMemBank(0);
  v = *pw;
  restore_flags(flags);
  if (v!=0x1234) { e=0x700000|v; goto quit; }
  if ((v=R_test1(0x5a5a))) { e=0x800000|v; goto quit; }
  if ((v=R_test1(0xa5a5))) { e=0x900000|v; goto quit; }
  if ((v=R_test1(0xffff))) { e=0xa00000|v; goto quit; }
  if ((v=R_test2())) { e=0xb00000|v; goto quit; }
  if ((v=R_test1(0x0000))) { e=0xc00000|v; goto quit; }

quit:
  SelectMemBank(0);
  k = MemErrAddr;
  (void)R_test1(0x0000);
  MemErrAddr = k;
  return e;
}


int TestMidi(void)
{
  int nM=0;

  if (!UseMidi) return 0;
  if (M_test(MPU1)==1) nM++;
  if (M_test(MPU2)==1) nM++;
  if (M_test(MPU3)==1) nM++;
  if (M_test(MPU4)==1) nM++;
  return nM;
}


int ResetTim(void)
{
  TimFsel = 3;	/* timer enable and clear, 61Hz */
  IrqCtrlSet_p(IQW_TIMEN|IQW_TIMCLR);
  IrqCtrlClr_p(IQW_TIMCLR);
  return 0;
}


static void ResetDtc3 (void)
{
    FPGArev = inb(DTC3ID+1) & 3;
    outb(0, WR_ANAL);
    CtrlSet(MASK_MIDE,0xff);	/* clear all, MIDI off */
    outb(0x00, IRQCTRL);	/* clear all */
    outb(0x00, IRQSTAT);	/* clear all */
}


static int SetupDtc3 (void)
{
  int i, retcode = 0;

    ResetBDM();
    switch (ResetTransp()) {
	case -1: printk(KERN_ERR "FPGA error in C012 access\n");
		 NumTransp=-1; break;
	case  0: printk(KERN_ERR "C012 access error\n");
		 NumTransp=-1; break;
	case  1: printk(KERN_INFO "C012 ok, no Transputers found\n"); break;
	case  2: printk(KERN_INFO "C012 ok, one Transputer found\n");
		 NumTransp=1; break;
	case  3: printk(KERN_INFO "C012 ok, two Transputers found\n");
		 NumTransp=2; break;
    }
    if ((MemState=SetupMem())) {
	printk(KERN_ERR "Error %#x at %#x in memory test\n",
		MemState,MemErrAddr);
	(void)ResetMem();
    }
    else {
	printk(KERN_INFO "Memory test ok\n");
	(void)ResetMem();
    }
    if (ResetTim()) {
	printk(KERN_INFO "Timer not working\n");
    }
    if (UseMidi) {
      CtrlSet(0x00,MASK_MIDE);	/* MIDI on(0) */
      if ((i=TestMidi())) {
 	printk(KERN_INFO "%d MIDI out found\n",i); NumMidi=i;
      }
      else
	printk(KERN_INFO "No MIDI out found\n");
    }

    if (NumTransp > 0) {
	if (link_init()) retcode = 1;
    }

    if (UseIrq > 0) {
  	  int result;
  	  /* enable and clear timer; this sets irq9 out=0 */
	  IrqCtrlSet_p(IQW_TIMEN|IQW_TIMCLR);
	  result = request_irq(UseIrq, dtc3_irq, SA_INTERRUPT,
	  	CARD_NAME, NULL);
	  if (result) {
	  	/* close 3-state output buffer for irq */
		CtrlSet(0x00,MASK_IRQON);
	  	printk(KERN_WARNING CARD_NAME": cannot use irq %d\n", UseIrq);
	  	UseIrq = -1;
	  	retcode = 1;
	  }
	  else {
	  	/* open 3-state output buffer for irq */
	  	printk(KERN_WARNING CARD_NAME": using irq %d\n", UseIrq);
		CtrlSet(MASK_IRQON,0x00);
	  }
	  IrqCtrlClr_p(IQW_TIMCLR);
    }

    return retcode;
}


static int DTCinitp2 (void)
{
  u_int e, ef;
  int retcode;

	retcode = 0;
	NumMidi = NumTransp = 0;
	UseIrq = IrqNum;
	
	if ((ef=DTCfound())==0) {
		printk(KERN_ERR "DTC3 not found\n");
		retcode=1; goto quit;
	}
	use_schedule = 1;

	if (ForceReset || (ef==1)) {
		if (ResetFPGA(1)) {
			printk(KERN_ERR "DTC3 card cannot be reset\n");
			retcode=1; goto quit;
		}
		FPGArev = -1;
		if (NoSetup) return 0;
		/* here: DONE low, INIT high */
		printk(KERN_WARNING "Reprogramming FPGA with default\n");
		e=FPGAConfTab();  /* Write data to device. */
		if ((e&(DONE|nERROR)) != (DONE|nERROR)) {
			printk(KERN_ERR "Error %#x programming FPGA\n", e);
			retcode=1; goto quit;
		}
		ef=2;
	}

	if (ef==2) {
	    FPGArev = inb(DTC3ID+1) & 3;
	    printk(KERN_WARNING "FPGA rev %d is programmed\n", FPGArev);
	    ResetDtc3();
	    if (NoSetup) return 0;
	    SetupDtc3();
	}
quit:
	return retcode;
}


/****************************************************************************/


int init_module(void)
{
	if (DTCinitp2()) return(-ENODEV);

	if ( register_chrdev( DTC3_MAJOR, CARD_NAME, &dtc3_fops ) )
	{
	  printk(KERN_ERR CARD_NAME": unable to get major %d for interface.\n",
		DTC3_MAJOR );
	  return -EIO;
	}

	if ( register_chrdev( LINK_MAJOR, LINK_NAME, &dtc2l_fops ) )
	{
	  printk(KERN_ERR CARD_NAME": unable to get major %d for link interface.\n",
		LINK_MAJOR );
	  return -EIO;
	}

	request_region(0x150,4,LINK_NAME);
	request_region(0x550,4,LINK_NAME);

	request_region(FPGAREPR,20,CARD_NAME);		/* 154..167 */
	request_region(FPGAREPR+0x400,4,CARD_NAME);	/* 554..557 */
	if (UseBDM)
		request_region(0x200,8,CARD_NAME);	/* 200..207 */
	if (UseMidi)
		request_region(MPU1,NumMidi*2,CARD_NAME);	/* 300..307 */

	proc_register(&proc_root, &dtc3_proc_entry);

	memset (min_in_use, 0, sizeof(min_in_use));
	return 0;
}


void cleanup_module( void) {

  if (MOD_IN_USE)
	printk(KERN_WARNING CARD_NAME": device busy, remove delayed\n");

  if (UseIrq > 0) {
	CtrlSet(0x00,MASK_IRQON);
	free_irq(UseIrq, NULL);
  }
  outb(0, WR_ANAL);
  ResetC012();
  ResetBDM();
  (void)ResetMem();
  outb(0x00, IRQCTRL);		/* clear all */
  outb(0x00, IRQSTAT);		/* clear all */
  CtrlSet(MASK_MIDE,0xff);	/* clear all, MIDI off */

  if (unregister_chrdev(DTC3_MAJOR, CARD_NAME) != 0) {
	printk(KERN_ERR "cleanup_module failed\n");
  } else {
	release_region(FPGAREPR,20);
	release_region(FPGAREPR+0x400,4);
	if (UseBDM)
		release_region(0x200,8);
	if (UseMidi)
		release_region(MPU1,NumMidi*2);
	if (NumTransp > 0) {
		unregister_chrdev( LINK_MAJOR, LINK_NAME );
		release_region(0x150,4);
		release_region(0x550,4);
	}
	proc_unregister(&proc_root, dtc3_proc_entry.low_ino);

	printk(KERN_INFO "DTC3 cleanup_module succeeded\n");
  }    
}


/****************************************************************************/

