
/* ================================================================= */
/*                                                                   */
/*		LOW-LEVEL FPGA interface                             */
/*                                                                   */
/* ================================================================= */

#ifdef __KERNEL__
# define PRINTK(a...)	printk(a)
#else
# include <stdio.h>
# define PRINTK(a...)	fprintf(stderr, ##a)
#endif
#define __NO_VERSION__
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/delay.h>

#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>

#include "dtc3_int.h"

#include "fpgabits.i"


static void nDelay(unsigned long d)
{
	udelay(d);
}

static void pDelay(unsigned long j)
{
	current->state = TASK_INTERRUPTIBLE;
	schedule_timeout(j/500);	/* j*10ms */
}


int ResetFPGA(int boot)
{
     int t;
     static void (*F)(unsigned long);

     F = boot? nDelay: pDelay;

     /* is FPGA already programmed? then, clear it */
     if (inb(FPGA)&DONE) {
       outb(0x5a, FPGAREPR);	/* enable clear, unlock flipflop */
       inb_p(0x80);
       if (inb(DTC3STAT) & MASK_rREPR) return 3;  /* check for enabled */
     }

     outb(0xff, FPGA_PGM);	/* Initialise the config memory. */

     F(1000);           /* Period to allow device to do internal clear. */
			/* This has to be a program loop, as interrupts are */
			/* now switched off, so we can't read the timers.*/
			/* Delayval must be calculated for each device.  */
			/* Alternatively we can assume worst case delay. */
			/* ie for largest device, and then add a margin. */

     outb(0x00, FPGA_PGM);	/* both output's required. */
     F(10000);

     if (inb(FPGA)&DONE) return 2;	/* error, DONE still high */

     /* wait for init pin going up */
     t = 1000;
     while (((inb(FPGA)&(DONE|INIT)) != INIT) && (t--))
	F(1000);

     return (t==0);	/* 0=ok, 1=timeout */
}


/* ----------------------------------------------------------------- */


int FPGAConfTab(void)
{
#ifdef DEBUG
  char s[80];
#endif
  int i, db, cnt;
  long flags;
  unsigned char *p = FPGABITS;
  unsigned char b, k = nERROR;

  db = 0;
#ifdef DEBUG
  *s = 0;
#endif
  for (cnt=0; cnt < sizeof(FPGABITS); cnt++)
     {
	b = *p++;
	save_flags(flags);
	cli();
	for (i=0; i<8; i++) {
	  outb_p((b&0x80? 0xff:0xfe),FPGA);
	  b<<=1;
	  k = inb(FPGA) & (DONE|nERROR);
	  if ((k&nERROR)==0) {
	  	restore_flags(flags);
#ifdef DEBUG
		printk("ERROR at %d\n",(cnt*8+i));
#endif
		return k;
	  }
	  if (k&DONE) {
#ifdef DEBUG
		if (db==0) sprintf(s,"DONE at %d",(cnt*8+i));
#endif
		db++; if (db > 4) goto quit;
	  }
	}
	restore_flags(flags);
     }
quit:
#ifdef DEBUG
  if (s) printk("%s\n",s);
#endif
  return k;
}


ssize_t fpga_write(struct file * file, const char * buf,
		        size_t count, loff_t *ppos)
{
	unsigned long p = *ppos;
	unsigned long end_mem = sizeof(FPGABITS);
	ssize_t written;

	if (p > end_mem)
  		return 0;
	if (count > (end_mem - p))
			count = end_mem - p;
	written = 0;
	if (copy_from_user(FPGABITS+p, buf, count))
		return -EFAULT;
	written += count;
	*ppos += written;
	return written;
}

