
/************************************************************************
*									*
*	Device Driver for DTC3 AV 1998                                  *
*									*
*************************************************************************/

#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/types.h>
#include <linux/ioport.h>
#include <linux/major.h>

#include <asm/io.h>
#include <asm/delay.h>
#include <asm/segment.h>

#include "dtc3_int.h"

/*
 * devices:	0=base and M68k
 *		6=stat
 *		16,17,18,19=MIDI ports
 */

#include <linux/string.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>

#include "uart401.h"

extern int in_use;			/* Total # of open devices */
extern int NumMidi;
extern int UseMidiIrq;
extern unsigned long TimCount, MidCount;
extern struct wait_queue *dtc3wq;

static int MidiBase[8] = { 0x300,0x302,0x304,0x306,0x330,-1,-1,-1 };
static int Hack[8] = { 0,0,0,0,0,0,0,0 };

static _mqs MQueue[8];
int MidQStat = 0;


/* ==================================================================== */
/*			RAW access to MPU-401				*/
/* ==================================================================== */

static inline int putcmd(int port, unsigned char b)
{
  int tout=0x7fff;

	while ((inb(port+1)&0x40)&&(tout--));
	if (tout) outb(b,port+1);
	  else return(-1);
	tout=0x7fff;
	while ((inb(port+1)&0x80)&&(tout--));
	if (tout && (inb(port)==0xfe)) return(0);
	  else return(-1);
}

#if 0
static int get401(int port)
{
 int i;

  while (1) {
    i = getdata(port);
    if (i != -1) return(i);
   }
}


static void sxcmd(int port, unsigned char c)
{
 static char Cb[] = { 0xf0,0x18,0x04,0x00 };
 int i;

  for (i=0; i<4; i++) putdata401(port, Cb[i]);
  putdata401(port, c);
}


static void eox(int port)
{
  putdata401(port, 0xf7);
}
#endif


static int sendcmd401(int port, unsigned char b)
{
 int ans;

  ans = putcmd(port, b);
  if (ans == ACK) return(ans);
   else if (ans != -1) {
     /* ungetnext401(ans);      put pending data on stack */
     while (1) {              /* check for more incoming data */
       ans = getdata(port);
       if (ans == ACK || ans == -1) return(ans);
	/* else ungetnext401(ans); */
     }
   }
  return(ans);
}


int repeat_cmd401(int port, unsigned char b)
{
 int i, j, m;

  for (i = 0; i < 10; i++){
    m = sendcmd401(port, b);
    if (m != -1) return(m);
     else {
       for (j = 0; j < 10; j++) {
	 getdata(port);
       }
     }
  }
  return(-1);
}


static void all_notes_off(int port, int nchans, int hack)
{
 int i;

  if ((nchans<0) || (nchans>16)) nchans=16;

  for(i = 0; i < nchans; i++){
    putdata(port, MODE_MESSAGE | i);
    putdata(port, 0x7f);
    putdata(port, 0);
    udelay(1000);
    putdata(port, MODE_MESSAGE | i);
    putdata(port, 0x79);
    putdata(port, 0);
    udelay(5000);
    putdata(port, MODE_MESSAGE | i);
    if (hack==MPU_ROLANDHACK)
	putdata(port, 0x78);
    else
	putdata(port, 0x7b);
    putdata(port, 0);
    current->state = TASK_INTERRUPTIBLE;
    schedule_timeout(2);
  }
}


/* ==================================================================== */


int dtc3_midi_open (unsigned int minor)
{
	_mqs *m;
	int i;

	m = &(MQueue[minor]);
	m->head = m->tail = m->flags = 0;
	memset(m->buf, 0, 256);
	m->port = MidiBase[minor];
	for (i=0; i<16; i++) m->map[i] = i;
	Hack[minor] = 0;
	MidQStat &= ~(1<<minor);
	if (UseMidiIrq)
		outb(MidQStat, IRQSTAT);
	return 0;
}

int dtc3_midi_release (unsigned int minor)
{
	MidQStat &= ~(1<<minor);
	if (UseMidiIrq)
		outb(MidQStat, IRQSTAT);
	return 0;
}


static char wbuf[256];

static char EMUsel[] = { 0xf0, 0x18, 4, 0, 3,
			 0, 2, 0, 0,
			 3, 2, 0, 0,
			 0xf7 };

ssize_t dtc3_midi_write(int minor, struct file *file, const char *buf,
		        size_t count, loff_t *ppos)
{
	_mqs *m;
	int fb, fc, written;
	long flags;

	if (count == 0) return 0;
	m = &(MQueue[minor]);

	save_flags(flags);
	cli();
	fc = (m->tail - m->head - 1) & 0xff;
	restore_flags(flags);
	if (fc == 0) {
		printk("Queue %d full\n", minor);
		return 0;
	}
	if (count > fc) count = fc;

	if (copy_from_user(wbuf, buf, count)) return -EFAULT;	

	if (Hack[minor]==MPU_PROTEUSHACK) {
		if ((count==8) && ((*wbuf&0xf0)==0xb0)) {
		  if ((wbuf[1]==0) && (wbuf[4]==32) && (wbuf[5]==0) &&
		  	((wbuf[3]&0xf0)==0xb0) && ((wbuf[6]&0xf0)==0xc0)) {
		  	int preset;
		  	if (fc < sizeof(EMUsel)) return 0;
		  	preset = ((wbuf[2]-80)<<7)+wbuf[7];
			EMUsel[7] = wbuf[0] & 15;
			EMUsel[11] = preset & 0x7f;
			EMUsel[12] = (preset>>7) & 0x7f;
			memcpy(wbuf, EMUsel, sizeof(EMUsel));
			count = sizeof(EMUsel);
		  }
		}
	}

	fb = 256 - m->head;
	memcpy(m->buf+m->head, wbuf, (count <= fb? count : fb));
	if (count > fb) {
		memcpy(m->buf, wbuf+fb, count-fb);
	}
	m->head += count;
	written = count;

	MidQStat |= (1<<minor);
	if (UseMidiIrq) {
		outb(MidQStat, IRQSTAT);
	}
	else {
		written = 0;
		while (m->head != m->tail) {
			register unsigned char b = m->buf[m->tail];
			if (putdata(m->port, b) < 0) break;
			m->tail++;
			written++;
		}
		if (m->head==m->tail)
			MidQStat &= ~(1<<minor);
	}

	return written;
}


int dtc3_midi_ioctl(int minor, struct file *file,
			    unsigned int cmd, unsigned long arg)
{
	char buf[16];
	int i, retval= 0;
	register _mqs *m = &(MQueue[minor]);

	switch (cmd) {
	  case DTC3_MPU_ALLNOTESOFF:
	  	all_notes_off(m->port, (arg? : 16), Hack[minor]);
	  	break;
	  case DTC3_MPU_CHANMAP:
		if (copy_from_user(buf, (void *)arg, 16)) return -EFAULT;
		for (i=0; i<16; i++) {
			m->map[i] = ((buf[i]&0xf0)==0? buf[i]:0xff);
		}
	  	break;
	  case DTC3_MPU_RDBYTE:
	  	retval = getdata(m->port);
	  	break;
	  case DTC3_MPU_WRBYTE:
		retval = putdata(m->port, arg);
	  	break;
	  case DTC3_MPU_HACK:
	  	Hack[minor] = arg;
		break;
	  default:
	  	return -EINVAL;
	}

	return retval;
}


/****************************************************************************/


void dtc3_irq (int irq, void *dev_id, struct pt_regs *regs)
{
  unsigned char b = inb(IRQSTAT);

  if (!(b & IQR_HWINT)) return;
  /* disable HW irq line */
  IrqCtrlSet(IQW_IRQACK);

  if (b & IQR_TIRQ) {
	/* clear timer flipflop */
	__asm__ __volatile__ ("
		inb	%%dx,%%al
		orb	%1,%%al
		outb	%%al,%%dx
		andb	%2,%%al
		outb	%%al,%%dx"
		:
		: "d"(IRQCTRL), "i"(IQW_TIMFFC), "i"(~IQW_TIMFFC)
		: "%eax" );
	TimCount++;
	wake_up_interruptible(&dtc3wq);
  }
  if (b & IQR_MIRQ) {
  	if (MidQStat) {
		register _mqs *m = MQueue;
		if (b & 1) {
			outb(m->buf[m->tail], m->port);
			m->tail++;
			if (m->tail == m->head) MidQStat &= (~1);
		}
		m++;
		if (b & 2) {
			outb(m->buf[m->tail], m->port);
			m->tail++;
			if (m->tail == m->head) MidQStat &= (~2);
		}
		m++;
		if (b & 4) {
			outb(m->buf[m->tail], m->port);
			m->tail++;
			if (m->tail == m->head) MidQStat &= (~4);
		}
		m++;
		if (b & 8) {
			outb(m->buf[m->tail], m->port);
			m->tail++;
			if (m->tail == m->head) MidQStat &= (~8);
		}
		MidCount++;
	}
	outb(MidQStat, IRQSTAT);
  }
  /* reenable HW irq line */
  IrqCtrlClr(IQW_IRQACK);
}


/****************************************************************************/

