
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/dtc3.h>

FILE *fs;
int fd;
struct stat fst;
int addr;

char nam[32], typ[32];
char dat[16], tim[16];

char hdr2[] = "unsigned char FPGABITS[] =\n{\n";

int header(void)
{
  unsigned short w;

  fseek(fs,0,SEEK_SET);
  w=fgetc(fs)<<8; w|=fgetc(fs);	/* big endian */
  if (w!=0x0009) return 1;
  fseek(fs,w,SEEK_CUR);
  w=fgetc(fs)<<8; w|=fgetc(fs);
  if (w!=0x0001) return 1;

  if (fgetc(fs)!='a') return 1;
  w=fgetc(fs)<<8; w|=fgetc(fs);	/* name len */
  fread(nam,1,w,fs);

  if (fgetc(fs)!='b') return 1;
  w=fgetc(fs)<<8; w|=fgetc(fs);	/* type len */
  fread(typ,1,w,fs);

  if (fgetc(fs)!='c') return 1;
  w=fgetc(fs)<<8; w|=fgetc(fs);	/* date len */
  fread(dat,1,w,fs);

  if (fgetc(fs)!='d') return 1;
  w=fgetc(fs)<<8; w|=fgetc(fs);	/* time len */
  fread(tim,1,w,fs);

  if (fgetc(fs)!='e') return 1;
  return 0;
}

int process(void)
{
  unsigned int i,k;
  char *buf;
  int e;

  if (header()) {
    printf("Unrecognized bitstream file\n");
    return 1;
  }

#ifdef DEBUG
  printf("FPGAappl=\"%s\"\n",nam);
  printf("FPGAtype=\"%s\"\n",typ);
  printf("FPGAdate=\"%s %s\"\n",dat,tim);
#endif

  fread(&k,1,4,fs);
  addr = (k&0xff); k>>=8;
  addr = (addr<<8)|(k&0xff); k>>=8;
  addr = (addr<<8)|(k&0xff); k>>=8;
  addr = (addr<<8)|(k&0xff);
  printf("Config file read is %#x bytes\n", addr);

  fd = open("/dev/dtc3fpga",O_WRONLY);
  if (fd < 0) {
	perror("/dev/dtc3fpga");
	return 1;
  }
#ifdef DEBUG
  printf("Loading FPGA bit table\n");
#endif
  buf = malloc(addr);
  if (fread(buf, 1, addr, fs) != addr) {
	printf("Error: partial file read\n");
	return 1;
  }
  if (write(fd, buf, addr) != addr) {
	printf("Error: partial file write\n");
	return 1;
  }
  close(fd);
  fd = open("/dev/dtc3fpga",O_WRONLY);

#ifdef DEBUG
  printf("Clearing FPGA device\n");
#endif
  if (ioctl(fd, DTC3_FPGA_CLR, 0x5a)) {
	perror("Clearing failed");
	return 1;
  }

  printf("Programming FPGA device\n");
  if ((e=ioctl(fd, DTC3_FPGA_PGM, 0)) != 0x60) {
	printf("Programming failed, err=%d\n",e);
	ioctl(fd, DTC3_FPGA_CLR, 0x5a);
	return 1;
  }

  ioctl(fd, DTC3_FPGA_APPL, nam);
  ioctl(fd, DTC3_FPGA_TYPE, typ);
  sprintf(buf,"%s %s",dat,tim);
  ioctl(fd, DTC3_FPGA_DATE, buf);

#ifdef DEBUG
  printf("Testing FPGA device\n");
#endif
  if (ioctl(fd, DTC3_FPGA_TEST, 0)) {
	perror("Testing failed");
	return 1;
  }

  free(buf);
  printf("Ok\n");
  return 0;
}


int main(int argc, char *argv[])
{
  if (argc<2) return 1;
  fs = fopen(argv[1],"r");
  if (fs==NULL) {
	printf("No such file %s\n",argv[1]);
	return 1;
  }
  fstat(fileno(fs), &fst);
  if (fst.st_size != 31065) {
  	printf("Bad length for 401E\n");
  	return 1;
  }
  process();
  if (fd>0) close(fd);
  fclose(fs);
  return 0;
}

